#include "Case.h"

#define FILE_ARCHIVE_EXTENSION "fors"

using namespace FS;

Case::Case(const QString& name, const unsigned int& version)
	:name(name),
	version(version)
{
}

Case::Case(const QString& directoryName) throw (CaseException)
{
	QRegExp regExp("^(.+) v([0-9]+)$");
	regExp.setMinimal(true);

	if (regExp.exactMatch(directoryName))
	{
		QStringList capturedTexts = regExp.capturedTexts();
		name = capturedTexts[1];
		version = capturedTexts[2].toUInt();
	}
	else
		throw CaseException("Case Exception: Object can't be created since the directory name is incorrectly formatted.");
}

const QString& Case::getName() const
{
	return name;
}

const unsigned int& Case::getVersion() const
{
	return version;
}

const QString Case::getDirectoryName() const
{
	return name + " v" + QString().setNum(version);
}

const QString Case::getFileArchiveName() const
{
	return getDirectoryName() + QString(".") + FILE_ARCHIVE_EXTENSION;
}

namespace FS
{
std::ostream& operator<<(std::ostream& outStream, const Case& case1)
{
	return outStream << case1.name.toStdString() << " v" << case1.version;
}
}
