/**
 * Case.h
 *
 *  Created on: May 7, 2009
 *      Author: barrygr
 */

#ifndef CASE_H_
#define CASE_H_

#include <QString>
#include <QStringList>
#include <QRegExp>
#include <QDir>
#include <stdexcept>
#include <iostream>

namespace FS
{

/**
 * Represents a single operation the a student may wish to learn, such as the removal of a wisdom tooth
 */
class Case
{
private:
	QString name;
	unsigned int version;

public:

	/**
	 * Run time exception class for Case
	 */
	class CaseException: public std::runtime_error
	{
		public:

			/**
			 * Constructor
			 */
			CaseException(const std::string& what) :
				runtime_error("Exception within Case: " + what)
				{
				}
	};

	/**
	 * Constructor
	 * /param The operation's name
	 * /param The operation's version number
	 */
	Case(const QString& name, const unsigned int& version);

	/*
	 * The directory name for this case,
	 * Expected in the format - "Operation Name vPerationNumber", e.g. "Remove Wisdom Tooth v1"
	 * Throws CaseException if this format is wrong
	 */
	Case(const QString& directoryName) throw (CaseException);

	const QString& getName() const;

	const unsigned int& getVersion() const;

	const QString getDirectoryName() const;

	const QString getFileArchiveName() const;

	friend std::ostream& operator<<(std::ostream& outStream, const Case& case1);
};

}
#endif //CASE_H_
