/*
 * DatabaseHandler.cpp
 *
 *  Created on: Oct 19, 2009
 *      Author: Daniel Stehn
 */
#include "DatabaseHandler.h"

DatabaseHandler* DatabaseHandler::m_pDataBaseHandlerInstance = 0;

/**
* protected constructor
*/
DatabaseHandler::DatabaseHandler()
{

}

/**
* protected destructor
*/
DatabaseHandler::~DatabaseHandler()
{

}

/**
* Initializes the member variables. Protected
*/
void DatabaseHandler::Initialize()
{
	m_forsDatabase = QSqlDatabase::addDatabase("QMYSQL");
	m_databaseIP 	= "";
	m_databaseName 	= "";
	m_username		= "";
	m_hashUsername	= "";
	m_hashPassword	= "";
	m_userId		= -1;
}

/**
* Gets the Anamnes information of a selected case. Public.
*/
bool DatabaseHandler::GetAnamnesText(const QString& rCaseName, const unsigned int& rCaseVersion, QString& rAnamnesText )
{
	bool bRetVal = false;

	if( m_forsDatabase.open() )
	{
		QSqlQuery query;
		query.prepare("select anamnes from forsCaseVersion join forsCase on forsCase.forsCaseId = forsCaseVersion.forsCaseId where forsCase.name = :caseName && forsCaseVersion.version = :version;");
		query.bindValue(":caseName", rCaseName );
		query.bindValue(":version", rCaseVersion );

		query.exec();
		query.next();
		rAnamnesText = query.value(0).toString();

		m_forsDatabase.close();
		bRetVal = true;
	}

	return bRetVal;
}


/**
* Gets the objectives information of a selected case. Public
*/
bool DatabaseHandler::GetCaseObjectives( const QString& rCaseName, const unsigned int& rCaseVersion, QString& rObjectives )
{
	bool bRetVal = false;

	if( m_forsDatabase.open() )
	{
		QSqlQuery query;
		query.prepare("select objectives from forsCaseVersion join forsCase on forsCase.forsCaseId = forsCaseVersion.forsCaseId where forsCase.name = :caseName && forsCaseVersion.version = :version;");
		query.bindValue(":caseName", rCaseName );
		query.bindValue(":version", rCaseVersion );

		query.exec();
		query.next();

		rObjectives = query.value(0).toString();

		m_forsDatabase.close();
		bRetVal = true;
	}

	return bRetVal;
}

/**
*  Gets a Id from a user name. Protected.
*/
int DatabaseHandler::GetUserIdFromDb( const QString& ruserName )
{
	int iRes = -1;

	// Create user in database and get their id. If this fails, then the userId is -1
	QSqlQuery query;

	// read user from database
	query.prepare("select userId from user where name = :name;");
	query.bindValue(":name", QVariant(ruserName) );

	if(query.exec())
	{
		if(query.next())
		{
			iRes = query.value(0).toInt();
		}
	}

	return iRes;
}


/**
* Gets the Id of a selected case. Public.
*/
unsigned int DatabaseHandler::GetSelectedCaseId( const QString& rName, const unsigned int& rVersion )
{
	unsigned int forsCaseVersionId = -1;

	if( m_forsDatabase.open() )
	{
		QSqlQuery query;
		query.prepare("select forsCaseVersionId from forsCase left join forsCaseVersion on forsCase.forsCaseId = forsCaseVersion.forsCaseId where name = :name && version = :version;");
		query.bindValue(":name", rName );
		query.bindValue(":version", rVersion );
		if(query.exec())
		{
			query.next();
			forsCaseVersionId = query.value(0).toInt();
		}

		m_forsDatabase.close();
	}

	return forsCaseVersionId;
}

/**
* Gets the server case list. Public.
*/
bool DatabaseHandler::GetServerCaseList( std::vector<CaseMembers>& rCaseList )
{
	bool bRetVal = false;

	if( m_forsDatabase.open() )
	{
		std::cout <<  "Reading list of cases form server" << std::endl;

		QSqlQuery query;
		query.exec("select name, max(version) from forsCase left join forsCaseVersion on forsCase.forsCaseId = forsCaseVersion.forsCaseId GROUP BY name;");

		while (query.next())
		{
			CaseMembers oData;
			oData.Name = query.value(0).toString();
			oData.version = query.value(1).toUInt();
			rCaseList.push_back(oData);
		}

		bRetVal = true;
		m_forsDatabase.close();
	}

	return bRetVal;
}


/**
* Gets the timestamp data for a selected case and timestamp. Public.
*/
bool DatabaseHandler::GetTimeStampData( const QString& rSelectedCase, const QString& selectedTimestamp, QByteArray& rResult )
{
	bool bRetval = false;

	if( m_forsDatabase.open() )
	{
		QSqlQuery query;

		query.prepare("select timeStampFile from userCase join user on user.userId = userCase.userId join forsCaseVersion on userCase.forsCaseVersionId = forsCaseVersion.forsCaseVersionId join forsCase on forsCase.forsCaseId = forsCaseVersion.forsCaseId where forsCase.name = :caseName && user.name = :userName && userCase.dateAdded = :timestamp;");
		query.bindValue(":userName", m_username );
		query.bindValue(":caseName", rSelectedCase );
		query.bindValue(":timestamp",selectedTimestamp);

		if(query.exec())
		{
			query.next();
			rResult = query.value(0).toByteArray();
		}

		bRetval = true;
		m_forsDatabase.close();
	}

	return bRetval;
}


/**
* Gets a list of timestamps that belongs to a selected case. Public.
*/
bool DatabaseHandler::GetTimeStampList( const QString& rSelectedCase, QStringList& stringList )
{
	bool bRetval = false;

	if( m_forsDatabase.open() )
	{
		QSqlQuery query;
		query.prepare("select dateAdded from userCase join user on user.userId = userCase.userId join forsCaseVersion on userCase.forsCaseVersionId = forsCaseVersion.forsCaseVersionId join forsCase on forsCase.forsCaseId = forsCaseVersion.forsCaseId where forsCase.name = :caseName && user.name = :userName;");
		query.bindValue(":userName", m_username );
		query.bindValue(":caseName", rSelectedCase );

		query.exec();
		while (query.next())
		{
			stringList<< query.value(0).toString();
		}

		bRetval = true;
		m_forsDatabase.close();
	}

	return bRetval;
}


/**
* Tries to open the database with login credentials. Protected.
*/
bool DatabaseHandler::OpenDbWithSavedCredentials()
{
	m_forsDatabase.setHostName(m_databaseIP);
	m_forsDatabase.setDatabaseName(m_databaseName);
	m_forsDatabase.setUserName(m_hashUsername);
	m_forsDatabase.setPassword(m_hashPassword);

	if(!m_forsDatabase.open())
	{
		return false;
	}

	return true;
}


/**
* Logs on to the database with the given credentials.
* If it fails to open the database or retrieve a ID for the user the function will throw an exception. Public.
*/
void DatabaseHandler::RegisterUser( const QString& rdatabaseIp, const QString& rdatabaseName, const QString& ruserName,
									const QString& rhashUsername, const QString& rhashPassword)
{
	//Try to save credentials. If any is empty Save will fail.
	if( false == SaveCredentials( rdatabaseIp, rdatabaseName, ruserName, rhashUsername, rhashPassword ) )
	{
		throw DatabaseHandlerException("Missing credentials information");
	}

	//Open database with saved credentials
	if( false == OpenDbWithSavedCredentials() )
	{
		throw DatabaseHandlerException("Please check user name and password.");
	}

	//Retrieve a ID for the username
	m_userId = GetUserIdFromDb( ruserName );

	if( m_userId == -1 )
	{
		throw DatabaseHandlerException("Failed to read user name from database");
	}

	//Close database
	m_forsDatabase.close();
}


/**
* Saves login credentials to member variables. Protected.
*/
bool DatabaseHandler::SaveCredentials(	const QString& rdatabaseIp, const QString& rdatabaseName, const QString& ruserName,
										const QString& rhashUsername, const QString& rhashPassword )
{
	if( rdatabaseIp.isEmpty() || rdatabaseName.isEmpty() || ruserName.isEmpty() ||
		rhashUsername.isEmpty() || rhashPassword.isEmpty() )
	{
		return false;
	}

	m_databaseIP 	= rdatabaseIp;
	m_databaseName 	= rdatabaseName;
	m_username		= ruserName;
	m_hashUsername	= rhashUsername;
	m_hashPassword	= rhashPassword;

	return true;
}



/**
* Saves the users performed exercise to the database.
* The function will throw an exception if it fails to save the information. Public.
*/
void DatabaseHandler::SavePerformedExercise( const QByteArray& rBytearray, const int& rforsCaseVersionId,
											 const std::vector<SegmentData>& rForbiddenSegmentedData,
											 const std::vector<SegmentData>& rSegmentedData )
{
	if(!m_forsDatabase.open())
	{
		throw DatabaseHandlerException("Failed to open database");
	}

	//1. Add recorded data to db
	QSqlQuery query;
	query.prepare("INSERT INTO userCase (userId, forsCaseVersionId, timeStampFile) "
				  "VALUES (:userId, :forsCaseVersionId, :timeStampFile)");
	query.bindValue(":userId", m_userId);
	query.bindValue(":forsCaseVersionId", rforsCaseVersionId);
	query.bindValue(":timeStampFile", rBytearray);

	if(query.exec())
		cout << "Added recorded data to database" << endl;
	else
	{
		throw DatabaseHandlerException("Failed to add recorded data to database");
	}

	//2. Get the ID of the added data
	query.prepare("SELECT LAST_INSERT_ID()");
	if(!query.exec())
	{
		throw DatabaseHandlerException("Failed to read last index from database");
	}

	query.next();
	int index = query.value(0).toInt();

	//3. Save Forbidden segment data to database
	vector<SegmentData>::const_iterator iterForbidden;

	for( iterForbidden = rForbiddenSegmentedData.begin(); iterForbidden != rForbiddenSegmentedData.end(); iterForbidden++  )
	{

		query.prepare("INSERT INTO userSegments (userCaseId, segmentId, drillingStep, voxelsRemoved, noVoxelsEqualToExpert) "
				"VALUES (:userCaseId, :segmentId, :drillingStep, :voxelsRemoved, :noVoxelsEqualToExpert)");
		query.bindValue(":userCaseId", index );
		query.bindValue(":segmentId", -(iterForbidden->SegmentId) );
		query.bindValue(":drillingStep", iterForbidden->DrillingStep );
		query.bindValue(":voxelsRemoved", iterForbidden->VoxelsRemoved );
		query.bindValue(":noVoxelsEqualToExpert", iterForbidden->noOfVoxelsEqualToExpert );

		if(query.exec())
			cout << "Added forbidden segment data to database " /*<<query.lastError().databaseText ().toStdString()*/ << endl;
		else
			cout << "No forbidden segments added to database" /*<< query.lastError().databaseText ().toStdString()*/ << endl;
	}

	//4. Save segment data to database
	vector<SegmentData>::const_iterator iter;

	for( iter = rSegmentedData.begin(); iter != rSegmentedData.end(); iter++  )
	{
		query.prepare("INSERT INTO userSegments (userCaseId, segmentId, drillingStep, voxelsRemoved, noVoxelsEqualToExpert) "
				"VALUES (:userCaseId, :segmentId, :drillingStep, :voxelsRemoved, :noVoxelsEqualToExpert)");
		query.bindValue(":userCaseId", index );
		query.bindValue(":segmentId", iter->SegmentId );
		query.bindValue(":drillingStep", iter->DrillingStep );
		query.bindValue(":voxelsRemoved", iter->VoxelsRemoved );
		query.bindValue(":noVoxelsEqualToExpert", iter->noOfVoxelsEqualToExpert );

		if(query.exec()){
			cout << "Added segment data to database" /*<< query.lastError().databaseText ().toStdString()*/ << endl;
		}
		else{
			cout << "No segments added to database" /*<< query.lastError().databaseText ().toStdString()*/ << endl;
		}
	}

	m_forsDatabase.close();
}


/**
* Returns a pointer to the single instance. Public.
*/
DatabaseHandler* DatabaseHandler::GetInstance()
{
	if( m_pDataBaseHandlerInstance == 0 )
	{
		m_pDataBaseHandlerInstance = new DatabaseHandler( );
		m_pDataBaseHandlerInstance->Initialize();
	}

	return m_pDataBaseHandlerInstance;
}


/**
* Destroys the single instance of DatabaseHandler.
* Should only be called when logging out from FsWisdom. Public
*/
void DatabaseHandler::DestroyInstance()
{
	if( m_pDataBaseHandlerInstance != 0 )
		{
			delete m_pDataBaseHandlerInstance;
			m_pDataBaseHandlerInstance = 0;
		}
}
