#ifndef GL_WINDOW_NODE_H
#define GL_WINDOW_NODE_H

#include <QtGui>
#include <GL/glew.h>
#include <QGLWidget>
#include <H3D/H3DWindowNode.h>
#include <H3D/Scene.h>
#include <H3D/Group.h>
#include <H3D/MouseSensor.h>
#include "ApplicationNode.h"
#include <H3D/INIFile.h>
#include <H3D/DEFNodes.h>

using namespace H3D;

namespace FS
{
  class ApplicationNode;

  /// \ingroup Nodes
  /// \class GLWindow
  /// \brief Standard OpenGL based window node.
  ///
  class GlWindowNode : public QGLWidget, public H3DWindowNode
  {

  public:
    Q_OBJECT

  public:

    /// Constructor.
    GlWindowNode(
    		ApplicationNode *applicationNode=0,
			const QDir& x3dFilePath=QDir(""),
    		Inst< SFInt32     > _width      = 0,
            Inst< SFInt32     > _height     = 0,
            Inst< SFBool      > _fullscreen = 0,
            Inst< SFBool      > _mirrored   = 0,
            Inst< RenderMode  > _renderMode = 0,
            Inst< SFViewpoint > _viewpoint  = 0,
            Inst< SFTime      > _time       = 0 );

    /// Destructor.
    ~GlWindowNode();

    virtual void swapBuffers();

    virtual void initWindow();

    virtual void initWindowHandler() { }

    virtual void setFullscreen( bool fullscreen );

    virtual void makeWindowActive();

    void mouseMoveEvent(QMouseEvent *event);
    void keyPressEvent(QKeyEvent *event);
    void mouseReleaseEvent(QMouseEvent *event);
    void mousePressEvent(QMouseEvent *event);
    void wheelEvent(QWheelEvent *event);

    void setLastRenderChild( X3DChildNode *n )
    {
      last_render_child = n;
    }

    void SetWidth(int width);
    void SetHeight(int height);

    void launchX3D(const QDir& x3dFilePath);

    Node* getSceneNode(const string& nodeName);

    void release();

    void startTimer();
    void stopTimer();

  public slots:
    /// refreshes the scene in a periodic fashion
    void updateScene();
    void DestroyOpenGLWindow();

  protected:
    void initializeGL();
    void paintGL();
    void resizeGL(int width, int height);

    /// The H3DNodeDatabase for this node.
    static H3DNodeDatabase database;

  private:
    void ClearAll();

	// the timer used to update the scene in the gl window
	auto_ptr<QTimer> timer;

	ApplicationNode* applicationNode;

	AutoRef< Scene > scene;
	AutoRef< Group > group;

	auto_ptr<X3D::DEFNodes> defNodes;

  };
}

#endif // GL_WINDOW_NODE_H
