/**
 * LocalCaseList.h
 *
 *  Created on: May 7, 2009
 *      Author: barrygr
 */

#ifndef LOCAL_CASE_LIST_H_
#define LOCAL_CASE_LIST_H_

#include "CaseList.h"

#include "ApplicationNode.h"
#include <QDir>
#include <QListWidget>

#include <QFile>
#include <QFileInfo>
#include <QtNetwork/QFtp>
#include <QtNetwork/QUrlInfo>
#include <iostream>
#include <stdexcept>
#include <memory>
#include <vector>

#include "../quazip/quazip.h"
#include "../quazip/quazipfile.h"


namespace FS
{

class ApplicationNode;

class LocalCaseList : public QObject, public CaseList
{
public:
	Q_OBJECT

private:
	const QDir& caseDirectory;
public:
	LocalCaseList(ApplicationNode *applicationNode, const QDir& caseDirectory, QFtp* Ftp);

	// This will delete any version of this case if it exists and replace it with that
	// referenced to via

	// for every case on the server, check if we need to
	// a) install the case for the first time since it isn't installed locally
	// b) install the latest version of the case locally

	void addLocally(const QDir& archiveFilePath);
	void addLocally(const Case& newCase);
	bool getX3dFilePathForSelectedRow(QDir& x3dFilePathForSelectedRow, QListWidget* const listWidget);
	const Case getCaseForSelectedRow(QListWidget* const listWidget);

	void ftpDownload(QString fileName);
	std::vector<QString> ftpDownloadVector;

	/**
	 * Run time exception class for LocalCaseList
	 */
	class LocalCaseListException: public std::runtime_error
	{
		public:

			/**
			 * Constructor
			 */
			LocalCaseListException(const std::string& what) :
				std::runtime_error("Exception within LocalCaseList: " + what)
				{
				}
	};

signals:
	void downloadComplete();

private slots:
	void ftpCommandFinished(int, bool error);

private:
    ApplicationNode* applicationNode;
	/**
	 * returns a string list with the directory name of each case stored herein
	 * e.g. "Remove Wisdom Tooth v1"
	 */
	QStringList getStringList() const;
	bool beginCaseInstallation(const Case& newCase, bool ignoreVersion);
	void unzipFile(const QDir& zipFilePath);
	bool removeDirectory(QDir &directory);

	QFtp* ftp;
	QFile *fileFtp;
};

}
#endif //LOCAL_CASE_LIST_H_
