/*
 * ProgressDialog.cpp
 *
 *  Created on: Jun 10, 2009
 *      Author: jockeka
 */

#include "ProgressDialog.h"
#include "ApplicationNode.h"
#include <QtGui/QCloseEvent>
#include <QFtp>
#include <QDir>
#include <QFile>
#include <QFileInfo>


using namespace FS;

ProgressDialog::ProgressDialog(ApplicationNode *a_applicationNode, const QString & labelText, const QString & cancelButtonText, int minimum, int maximum, QWidget *parent)
:QProgressDialog(labelText,cancelButtonText,minimum,maximum,parent)
{
	applicationNode = a_applicationNode;
}

ProgressDialog::~ProgressDialog()
{
}

void ProgressDialog::setMax(int newMax)
{
	maxValue = newMax;
	setMaximum(newMax);
}

void ProgressDialog::setVal(int newValue)
{
	setValue(newValue);
}

void ProgressDialog::setValueDownloadingCases(int newValue)
{
	setValue(maxValue-newValue);
}

void ProgressDialog::setNewText(QString text)
{
	setLabelText(text);
}

void ProgressDialog::closeEvent(QCloseEvent *event)
{
	//This event is sent when user press the Close-button (X-button).
	if(applicationNode == NULL)
	{
		event->ignore();
		return;
	}

	std::cout << "Progress dialog closed" << std::endl;
	cancelFtpDownloading();
	event->accept();
}

void ProgressDialog::cancelFtpDownloading()
{
	//The Cancel-button on the progress-dialog is connected to this SLOT.
	// Stop ftp transfer
	QFtp* ftp = applicationNode->getFtp();
	ftp->abort();
	ftp->close();
	applicationNode->setFtpAborted(true);

	// Remove directories
	if(!removeDirectories())
		std::cout << "Not able to remove empty case directories" << std::endl;
}

bool ProgressDialog::removeDirectories(void)
{
	QDir directory = QDir(applicationNode->getCaseDirectory().absolutePath());
	bool isSuccessful = true;
	if (directory.exists())
		{
			QFileInfoList entries = directory.entryInfoList(QDir::NoDotAndDotDot | QDir::Hidden | QDir::Dirs);
			int count = entries.size();
			if(count == 0)
				return true;
			for (int i = 0; ((i < count) && isSuccessful); i++)
			{
				QFileInfo entryInfo = entries[i];
				QString path = entryInfo.absoluteFilePath();
				QDir subDirectory = QDir(path);
				QFileInfoList subEntries = subDirectory.entryInfoList(QDir::NoDotAndDotDot | QDir::Hidden | QDir::Dirs | QDir::Files);
				int subCount = subEntries.size();
				if(subCount == 0)
					isSuccessful = subDirectory.rmdir(subDirectory.absolutePath());		// Remove empty directory
			}
		}
	return isSuccessful;

}

