#ifndef Q_DEBUG_H
#define Q_DEBUG_H

#include <iostream>
#include <streambuf>
#include <string>
#include <fstream>
#include <QDir>

class QDebugStream : public std::basic_streambuf<char>
{
public:
	QDebugStream(std::ostream &stream, QTextEdit* text_edit, const QDir& outputFilePath) : m_stream(stream), outputFilePath(outputFilePath)
	{
		log_window = text_edit;
		m_old_buf = stream.rdbuf();
		stream.rdbuf(this);

		std::ofstream fileOutput;
		fileOutput.open(outputFilePath.path().toStdString().c_str(), ios::out);
		fileOutput << "Start" << endl;
		fileOutput.close();

		count = 0;
	}

	~QDebugStream()
	{
		// output anything that is left
		if (!m_string.empty())
		{
			log_window->append(m_string.c_str());

			std::ofstream fileOutput;
			fileOutput.open(outputFilePath.path().toStdString().c_str(), ios::out | ios::app);

			if(fileOutput.is_open())
			{
				fileOutput << count<<m_string.c_str() << std::endl;
				count++;
				fileOutput.close();
			}
		}

		m_stream.rdbuf(m_old_buf);

		cout << "Destructing QDebugStream" << endl;
	}

protected:
	virtual int_type overflow(int_type v)
	{
		if (v == '\n')
		{
			log_window->append(m_string.c_str());

			std::ofstream fileOutput;
			fileOutput.open(outputFilePath.path().toStdString().c_str(), ios::out | ios::app);

			if(fileOutput.is_open())
			{
				fileOutput << count << m_string.c_str() << std::endl;
				count++;
				fileOutput.close();
			}
			m_string.erase(m_string.begin(), m_string.end());
		}
		else
			m_string += v;

		return v;
	}

	virtual std::streamsize xsputn(const char *p, std::streamsize n)
	{
		m_string.append(p, p + n);

		std::ofstream fileOutput;

		std::string::size_type pos = 0;
		while (pos != std::string::npos)
		{
			pos = m_string.find('\n');
			if (pos != std::string::npos)
			{
				std::string tmp(m_string.begin(), m_string.begin() + pos);
				log_window->append(tmp.c_str());

				fileOutput.open(outputFilePath.path().toStdString().c_str(), ios::out | ios::app);

				if(fileOutput.is_open())
				{
					fileOutput <<count<< tmp << std::endl;
					count++;
					fileOutput.close();
				}

				m_string.erase(m_string.begin(), m_string.begin() + pos + 1);
			}
		}

		return n;
	}

private:
	std::ostream &m_stream;
	std::streambuf *m_old_buf;
	std::string m_string;
	QTextEdit* log_window;
	const QDir outputFilePath;
	int count;

};

#endif
