#include "RemoveWisdomToothWindow.h"

using namespace FS;

RemoveWisdomToothWindow::RemoveWisdomToothWindow(ApplicationNode *applicationNode, bool playback, QWidget* parent)
: QMainWindow(parent),
applicationNode(NULL),
volumeModel(NULL),
room(NULL),
fakeCamera(NULL),
patient(NULL),
teeth(NULL),
head(NULL),
gumCut(NULL),
teethGum(NULL),
lowerTeeth(NULL),
upperTeeth(NULL),
floorOfMouth(NULL),
tongue(NULL),
hook(NULL),
aDrillableNode(NULL),
hapticDeviceNode(NULL),
drillingStepsNode(NULL),
materialSegmentationModel(NULL),
forbiddenSegmentationModel(NULL),
highlightNode(NULL),
playbackNode(NULL),
defaultValues(NULL),
saveVolume(NULL),
showHead(NULL),
showRoom(NULL),
drillable(NULL),
showTeeth(NULL),
deviceShape(NULL),
vibrations(NULL),
stateMachine(NULL),
loggingNode(NULL),
isPlayback(false),
orgAngle(0.0),
orgRot(0.0, 0.0, 0.0)
{
	this->applicationNode = applicationNode;

	ui.setupUi(this);

	ui.statisticsTableWidget->horizontalHeader()->setDefaultSectionSize(173);
	ui.stateTableWidget->horizontalHeader()->setDefaultSectionSize(630);
	//	ui.stateTableWidget->verticalHeader()->setDefaultSectionSize(50);

	//    ui.statisticsTableWidget->verticalHeader()->setDefaultSectionSize(50);


	frameRateLabel.reset(new QLabel("0", this));
	hapticRateLabel.reset(new QLabel("0", this));

	statusBar()->addWidget(new QLabel("Frame rate: ", this));
	statusBar()->addWidget(frameRateLabel.get());
	statusBar()->addWidget(new QLabel("Haptic rate: ", this));
	statusBar()->addWidget(hapticRateLabel.get());

	QObject::connect(ui.highlightUserButton, SIGNAL(clicked()),
			this, SLOT(highlightUser()));

	QObject::connect(ui.highlightExpertButton, SIGNAL(clicked()),
			this, SLOT(highlightExpert()));

	QObject::connect(ui.highlightForbiddenButton, SIGNAL(clicked()),
			this, SLOT(highlightForbidden()));

	QObject::connect(ui.decrementCurrentDrillingStepButton, SIGNAL(clicked()),
			this, SLOT(decrementCurrentDrillingStep()));

	QObject::connect(ui.incrementCurrentDrillingStepButton, SIGNAL(clicked()),
			this, SLOT(incrementCurrentDrillingStep()));

	QObject::connect(ui.clearCurrentExpertDrillingStepButton, SIGNAL(clicked()),
			this, SLOT(clearCurrentDrillingStep()));

	QObject::connect(ui.addExpertDrillingStepButton, SIGNAL(clicked()),
			this, SLOT(addDrillingStep()));

	QObject::connect(ui.saveExpertDrillingStepsToFileButton, SIGNAL(clicked()),
			this, SLOT(saveExpertDrillingSteps()));

	QObject::connect(ui.editExpertDrillingStepCheckBox, SIGNAL(stateChanged(int)), this, SLOT(enterEditExpertDrillingStepsMode(int)));

	QObject::connect(ui.exitPushButton, SIGNAL(clicked ()), applicationNode, SLOT(exitX3DWithStandardGuiWindow ()));

	QObject::connect(ui.headButton, SIGNAL(clicked ()), this, SLOT(toggleShowHideHead ()));
	QObject::connect(ui.teethButton, SIGNAL(clicked ()), this, SLOT(toggleShowHideTeeth ()));
	QObject::connect(ui.toolButton, SIGNAL(clicked ()), this, SLOT(toggleShowHideTool ()));

	QObject::connect(ui.selectDrillButton, SIGNAL(clicked ()), this, SLOT(selectDrill ()));
	QObject::connect(ui.selectElevatorButton, SIGNAL(clicked ()), this, SLOT(selectElevator ()));


	initialize(playback);
}

void RemoveWisdomToothWindow::initialize(bool playback)
{
	cout<<"Initializing RemoveWisdomToothWindow"<<endl;

	materialSegmentationModel 	= static_cast <MaterialSegmentationModel*>		(applicationNode->getSceneNode("SM"));
	volumeModel 				= static_cast <VolumeModel*>					(applicationNode->getSceneNode("VM"));
	fakeCamera					= static_cast <Transform*> 						(applicationNode->getSceneNode("fakecamera"));
	room						= static_cast <Transform*> 						(applicationNode->getSceneNode("ROOM"));
	patient						= static_cast <Transform*> 						(applicationNode->getSceneNode("THE_PATIENT"));
	teeth						= static_cast <Transform*> 						(applicationNode->getSceneNode("TEETH"));
	head						= static_cast <Transform*> 						(applicationNode->getSceneNode("HEAD"));
	gumCut						= static_cast <Transform*> 						(applicationNode->getSceneNode("GUMCUT"));
	lowerTeeth					= static_cast <Transform*> 						(applicationNode->getSceneNode("LOWERTEETH"));
	upperTeeth					= static_cast <Transform*> 						(applicationNode->getSceneNode("UPPERTEETH"));
	teethGum					= static_cast <Transform*> 						(applicationNode->getSceneNode("TEETHGUM"));
	floorOfMouth				= static_cast <Transform*> 						(applicationNode->getSceneNode("FLOOROFMOUTH"));
	tongue						= static_cast <Transform*>						(applicationNode->getSceneNode("TONGUE"));
	hook						= static_cast <Transform*> 						(applicationNode->getSceneNode("HOOK"));
	aDrillableNode				= static_cast <ADrillableNode*> 				(applicationNode->getSceneNode("DN"));
	drillingStepsNode			= static_cast <DrillingStepsNode*> 				(applicationNode->getSceneNode("DS"));
	highlightNode				= static_cast <HighlightNode*> 					(applicationNode->getSceneNode("HN"));
	playbackNode				= static_cast <AVolumePlaybackNode*>			(applicationNode->getSceneNode("VPN"));
	defaultValues				= static_cast <DefaultValues*>					(applicationNode->getSceneNode("DV"));
	showHead					= static_cast <Switch*>							(applicationNode->getSceneNode("showHead"));
	showRoom					= static_cast <Switch*>							(applicationNode->getSceneNode("showRoom"));
	drillable					= static_cast <Switch*>							(applicationNode->getSceneNode("DRILLABLE"));
	showTeeth					= static_cast <Switch*>							(applicationNode->getSceneNode("showTeeth"));
	deviceShape					= static_cast <Switch*>							(applicationNode->getSceneNode("deviceShape"));
	vibrations					= static_cast <Switch*>							(applicationNode->getSceneNode("VIB"));
	saveVolume					= static_cast <SaveVolume*>						(applicationNode->getSceneNode("SV"));
	loggingNode					= static_cast <LoggingNode*>					(applicationNode->getSceneNode("LN"));
	stateMachine				= static_cast <StateMachine*>					(applicationNode->getSceneNode("ST"));
	forbiddenSegmentationModel 	= static_cast <ForbiddenSegmentationModel*>		(applicationNode->getSceneNode("FM"));
	hapticDeviceNode 			= static_cast <PhantomDevice*>					(applicationNode->getSceneNode("HDEV"));

	updater_label_frameRateField.reset();
	updater_label_frameRateField.reset(
			new Updater_QLabel_SFFloat(frameRateLabel.get(),aDrillableNode?aDrillableNode->currentFPS.get():NULL));

	updater_label_hapticRateLabel.reset();
	updater_label_hapticRateLabel.reset(
			new Updater_QLabel_SFInt32(hapticRateLabel.get(),hapticDeviceNode?hapticDeviceNode->hapticsRate.get():NULL));

	/*current state field in the old GUI tabs*/
	updater_label_stateField.reset();
	updater_label_stateField.reset(
			new Updater_QLabel_SFInt32(ui.state,stateMachine?stateMachine->stateField.get():NULL));

	updater_label_stateField_2.reset();
	updater_label_stateField_2.reset(
			new Updater_QLabel_SFInt32(ui.state_2,stateMachine?stateMachine->stateField.get():NULL));

	//Changeable parameter routed from and to the UI
	updater_slider_materialHardnessField.reset();
	updater_slider_materialHardnessField.reset(new Updater_QSlider_MaterialHardness(ui,materialSegmentationModel,materialSegmentationModel?materialSegmentationModel->segmentHardnessField.get():NULL));

	updater_slider_cutRadiusField.reset();
	updater_slider_cutRadiusField.reset(new Updater_QSlider_SFFloat(ui.cutRadiusSlider,ui.cutRadius,aDrillableNode?aDrillableNode->cutRadius.get():NULL,0.005,0.0001));

	updater_slider_drillSpeedMaxField.reset();
	updater_slider_drillSpeedMaxField.reset(new Updater_QSlider_SFFloat(ui.drillSpeedMaxSlider,ui.drillSpeedMax,defaultValues?defaultValues->drillSpeedMaxField.get():NULL,200.0,1.0));

	//user Mode
	updater_button_nextStateField.reset();
	updater_button_nextStateField.reset(new Updater_QButton_NextState(ui.nextStateButton,stateMachine));

	updater_button_removeCrownByNameField.reset();
	updater_button_removeCrownByNameField.reset(new Updater_QButton_RemovePartitionByName(ui.removeCrownButton,volumeModel,"Crown"));

	updater_button_removeRootByNameField.reset();
	updater_button_removeRootByNameField.reset(new Updater_QButton_RemovePartitionByName(ui.removeRootButton,volumeModel,"Root"));

	updater_button_removeOtherByNameField.reset();
	updater_button_removeOtherByNameField.reset(new Updater_QButton_RemovePartitionByName(ui.removeOtherButton,volumeModel,"Other teeth"));


	//Information only displayed
	updater_label_drillSpeedField.reset();
	updater_label_drillSpeedField.reset(new Updater_QLabel_SFFloat(ui.drillSpeed,aDrillableNode?aDrillableNode->drillSpeed.get():NULL));
	updater_label_drillPosField.reset();
	updater_label_drillPosField.reset(new Updater_QLabel_SFVec3f(ui.drillPos,hapticDeviceNode?hapticDeviceNode->trackerPosition.get():NULL,0,ui,stateMachine));
	updater_label_forceField.reset();
	updater_label_forceField.reset(new Updater_QLabel_SFVec3f(ui.force,hapticDeviceNode?hapticDeviceNode->force.get():NULL,1,ui,stateMachine));


	setupStatistics(); //must be done before we add the updater
	setupObjectives();

	updater_tableWidget_statisticsField.reset();
	updater_tableWidget_statisticsField.reset(new Updater_QTableWidget(
			ui.statisticsTableWidget,
			materialSegmentationModel?materialSegmentationModel->noOfVoxelsBoredByUser.get():NULL,forbiddenSegmentationModel?forbiddenSegmentationModel->noOfVoxelsBoredByUser.get():NULL,drillingStepsNode));

	//Model configuration configured via radio buttons and push buttons
	updater_label_viewTranslationField.reset();
	updater_label_viewTranslationField.reset(new Updater_QLabel_SFTranslation(ui,ui.viewTranslation,ui.viewRadio,fakeCamera?fakeCamera->translation.get():NULL,defaultValues?defaultValues->translationZoomFactorField.get():NULL));

	updater_label_viewRotationField.reset();
	updater_label_viewRotationField.reset(new Updater_QLabel_SFRotation(ui, ui.viewRotation,ui.viewRadio,fakeCamera?fakeCamera->rotation.get():NULL,defaultValues?defaultValues->rotationZoomFactorField.get():NULL,0));

	updater_label_patTranslationField.reset();
	updater_label_patTranslationField.reset(new Updater_QLabel_SFTranslation(ui,ui.patTranslation,ui.patRadio,patient?patient->translation.get():NULL,defaultValues?defaultValues->translationZoomFactorField.get():NULL));

	updater_label_patRotationField.reset();
	updater_label_patRotationField.reset(new Updater_QLabel_SFRotation(ui,ui.patRotation,ui.patRadio,patient?patient->rotation.get():NULL,defaultValues?defaultValues->rotationZoomFactorField.get():NULL,0));

	updater_label_teethTranslationField.reset();
	updater_label_teethTranslationField.reset(new Updater_QLabel_SFTranslation(ui,ui.teethTranslation,ui.teethRadio,teeth?teeth->translation.get():NULL,defaultValues?defaultValues->translationZoomFactorField.get():NULL));

	updater_label_teethRotationField.reset();
	updater_label_teethRotationField.reset(new Updater_QLabel_SFRotation(ui,ui.teethRotation,ui.teethRadio,teeth?teeth->rotation.get():NULL,defaultValues?defaultValues->rotationZoomFactorField.get():NULL,0));

	updater_label_headTranslationField.reset();
	updater_label_headTranslationField.reset(new Updater_QLabel_SFTranslation(ui,ui.headTranslation,ui.headRadio,head?head->translation.get():NULL,defaultValues?defaultValues->translationZoomFactorField.get():NULL));

	updater_label_headRotationField.reset();
	updater_label_headRotationField.reset(new Updater_QLabel_SFRotation(ui,ui.headRotation,ui.headRadio,head?head->rotation.get():NULL,defaultValues?defaultValues->rotationZoomFactorField.get():NULL,0));

	updater_label_gumCutTranslationField.reset();
	updater_label_gumCutTranslationField.reset(new Updater_QLabel_SFTranslation(ui,ui.gumCutTranslation,ui.gumCutRadio,gumCut?gumCut->translation.get():NULL,defaultValues?defaultValues->translationZoomFactorField.get():NULL));

	updater_label_gumCutRotationField.reset();
	updater_label_gumCutRotationField.reset(new Updater_QLabel_SFRotation(ui,ui.gumCutRotation,ui.gumCutRadio,gumCut?gumCut->rotation.get():NULL,defaultValues?defaultValues->rotationZoomFactorField.get():NULL,0));

	updater_label_lowerTeethTranslationField.reset();
	updater_label_lowerTeethTranslationField.reset(new Updater_QLabel_SFTranslation(ui,ui.lowerTeethTranslation,ui.lowerTeethRadio,lowerTeeth?lowerTeeth->translation.get():NULL,defaultValues?defaultValues->translationZoomFactorField.get():NULL));

	updater_label_lowerTeethRotationField.reset();
	updater_label_lowerTeethRotationField.reset(new Updater_QLabel_SFRotation(ui,ui.lowerTeethRotation,ui.lowerTeethRadio,lowerTeeth?lowerTeeth->rotation.get():NULL,defaultValues?defaultValues->rotationZoomFactorField.get():NULL,0));

	updater_label_upperTeethTranslationField.reset();
	updater_label_upperTeethTranslationField.reset(new Updater_QLabel_SFTranslation(ui,ui.upperTeethTranslation,ui.upperTeethRadio,upperTeeth?upperTeeth->translation.get():NULL,defaultValues?defaultValues->translationZoomFactorField.get():NULL));

	updater_label_upperTeethRotationField.reset();
	updater_label_upperTeethRotationField.reset(new Updater_QLabel_SFRotation(ui,ui.upperTeethRotation,ui.upperTeethRadio,upperTeeth?upperTeeth->rotation.get():NULL,defaultValues?defaultValues->rotationZoomFactorField.get():NULL,0));

	updater_label_teethGumTranslationField.reset();
	updater_label_teethGumTranslationField.reset(new Updater_QLabel_SFTranslation(ui,ui.teethGumTranslation,ui.teethGumRadio,teethGum?teethGum->translation.get():NULL,defaultValues?defaultValues->translationZoomFactorField.get():NULL));

	updater_label_teethGumRotationField.reset();
	updater_label_teethGumRotationField.reset(new Updater_QLabel_SFRotation(ui,ui.teethGumRotation,ui.teethGumRadio,teethGum?teethGum->rotation.get():NULL,defaultValues?defaultValues->rotationZoomFactorField.get():NULL,0));

	updater_label_tongueTranslationField.reset();
	updater_label_tongueTranslationField.reset(new Updater_QLabel_SFTranslation(ui,ui.tongueTranslation,ui.tongueRadio,tongue?tongue->translation.get():NULL,defaultValues?defaultValues->translationZoomFactorField.get():NULL));

	updater_label_tongueRotationField.reset();
	updater_label_tongueRotationField.reset(new Updater_QLabel_SFRotation(ui,ui.tongueRotation, ui.tongueRadio,tongue?tongue->rotation.get():NULL,defaultValues?defaultValues->rotationZoomFactorField.get():NULL,0));

	updater_label_floorOfMouthTranslationField.reset();
	updater_label_floorOfMouthTranslationField.reset(new Updater_QLabel_SFTranslation(ui,ui.floorOfMouthTranslation,ui.floorOfMouthRadio,floorOfMouth?floorOfMouth->translation.get():NULL,defaultValues?defaultValues->translationZoomFactorField.get():NULL));

	updater_label_floorOfMouthRotationField.reset();
	updater_label_floorOfMouthRotationField.reset(new Updater_QLabel_SFRotation(ui,ui.floorOfMouthRotation, ui.floorOfMouthRadio,floorOfMouth?floorOfMouth->rotation.get():NULL,defaultValues?defaultValues->rotationZoomFactorField.get():NULL,0));

	updater_label_hookTranslationField.reset();
	updater_label_hookTranslationField.reset(new Updater_QLabel_SFTranslation(ui,ui.hookTranslation,ui.hookRadio,hook?hook->translation.get():NULL,defaultValues?defaultValues->translationZoomFactorField.get():NULL));

	updater_label_hookRotationField.reset();
	updater_label_hookRotationField.reset(new Updater_QLabel_SFRotation(ui,ui.hookRotation, ui.hookRadio,hook?hook->rotation.get():NULL,defaultValues?defaultValues->rotationZoomFactorField.get():NULL,0));

	updater_slider_translationZoomFactorField.reset();
	updater_slider_translationZoomFactorField.reset(new Updater_QSlider_SFFloat(ui.translationZoomFactorSlider,ui.translationZoomFactor,defaultValues?defaultValues->translationZoomFactorField.get():NULL,0.005,0.0001));

	updater_slider_rotationZoomFactorField.reset();
	updater_slider_rotationZoomFactorField.reset(new Updater_QSlider_SFFloat(ui.rotationZoomFactorSlider,ui.rotationZoomFactor,defaultValues?defaultValues->rotationZoomFactorField.get():NULL,0.05,0.001));

	//User tab
	updater_label_playbacktime_2.reset();
	updater_label_playbacktime_2.reset(new Updater_QLabel_SFFloat(ui.playbacktime_2,playbackNode?playbackNode->playbackTimeField.get():NULL));

	updater_button_playstop_2.reset();
	updater_button_playstop_2.reset(new Updater_QButton_Playback(ui.playstop_2,playbackNode?playbackNode->isPlay.get():NULL));

	updater_button_reset_2.reset();
	updater_button_reset_2.reset(new Updater_QButton_SFBoolPB(ui.reset_2,playbackNode?playbackNode->resetPlaybackTimeField.get():NULL));

	updater_button_rewind_2.reset();
	updater_button_rewind_2.reset(new Updater_QButton_SFFloat(ui.rw_2,playbackNode?playbackNode->directionFastPlayback.get():NULL,-1,playbackNode));

	updater_button_fastforward_2.reset();
	updater_button_fastforward_2.reset(new Updater_QButton_SFFloat(ui.ffw_2,playbackNode?playbackNode->directionFastPlayback.get():NULL,1,playbackNode));

	updater_progressbar_playbackProgressBar_2.reset();
	updater_progressbar_playbackProgressBar_2.reset(new Updater_QProgressBar_PlaybackProgressBar(ui.progressBar_2,playbackNode?playbackNode->playbackTimeField.get():NULL,playbackNode,volumeModel));

	//	updater_label_fractionField_2.reset(new Updater_QLabel_SFFloat(ui.fraction_2,drillingStepsNode->fractionOfVoxelsDrilledByUserToExpertForCurrentDrillingStepField.get()));
	updater_label_teethRotationField_2.reset();
	updater_label_teethRotationField_2.reset(new Updater_QLabel_SFRotation(ui,ui.teethRotation,ui.teethRadio,teeth?teeth->rotation.get():NULL,defaultValues?defaultValues->rotationZoomFactorField.get():NULL,1));
	/**The select drill and select elevator button is connected to selectDrill() and selectElevator() functions (see above).
	 * That will take care of updating the x3d file and python script of current tool.
	 * But we also need to listen for changes in deviceShape (a Switch in x3d) done in python (that is the user
	 * uses the keyboard to change tool). Therefore we need to set up a route from deviceShape to our GUI-buttons.
	 * Both listening and setting of deviceShape could be taken care of in the Updater_QButton_SFBoolPB-class
	 * but for now setting of deviceShape in x3d is taken care of in selectDrill() and selectElevator().
	 * TODO: Maybe possible to listen for changes in field in a different way?
	 */
	update_button_drill.reset();
	update_button_drill.reset(new Updater_QButton_SFInt32_Drill_Elevator(ui,deviceShape?deviceShape->whichChoice.get():NULL));

	this->isPlayback = playback;
	if (playback)
	{
		enablePlayback();
	}
	else
	{
		disablePlayback();
	}

	//Save the original rotation of teeth
	SFRotation* teethRot = teeth->rotation.get();
	orgRot = teethRot->getValue().axis;
	orgAngle = teethRot->getValue().angle;

	//Be sure that buttons are in default-state (on user tab at least).
	if (aDrillableNode->hasPedal.get()->getValue())
	{
		//If pedals are connected the GUI buttons for changing tools should be disabled.
		//Though you can still use the keyboard ('h' and '<space>'.) This seems to be synched in an acceptable way.
		ui.selectDrillButton->setEnabled(false);
		ui.selectElevatorButton->setEnabled(false);
	}
	else
	{
		ui.selectDrillButton->setChecked(true);
		ui.selectElevatorButton->setChecked(false);
	}

	ui.headButton->setChecked(true);
	ui.teethButton->setChecked(true);
	ui.toolButton->setChecked(true);
	ui.highlightUserButton->setChecked(false);
	ui.highlightExpertButton->setChecked(false);
	ui.highlightForbiddenButton->setChecked(false);
	ui.rotationPlusY_2->setEnabled(false);
	ui.rotationPlusZ_2->setEnabled(false);
	ui.rotationMinusY_2->setEnabled(false);
	ui.rotationMinusZ_2->setEnabled(false);
	ui.progressBar_2->setValue(0);

	//Need to reset drillspeed. For some reason it will still have a value > 0 if starting
	//a case after exiting a case with the drill on. And it is not enough to reset the field
	//in ADrillableNode.cpp for some reason. Is the python-classes not reseted on exit?
	aDrillableNode->drillSpeed.get()->setValue(0);
	vibrations->whichChoice.get()->setValue(0);

	//Update ui according to state
	toggleShowHideHead();
	toggleShowHideTeeth();
	toggleShowHideTool();

	cout<<"Initialized RemoveWisdomToothWindow"<<endl;
}

RemoveWisdomToothWindow::~RemoveWisdomToothWindow()
{
	cout << "Destroying RemoveWisdomToothWindow"<<endl;
}

void RemoveWisdomToothWindow::DestroyRemoveWisdomToothWindow()
{
	this->destroy(true, true );
}

void RemoveWisdomToothWindow::decrementCurrentDrillingStep()
{
	if(drillingStepsNode)
		drillingStepsNode->decrementCurrentDrillingStep();
}

void RemoveWisdomToothWindow::incrementCurrentDrillingStep()
{
	if(drillingStepsNode)
		drillingStepsNode->incrementCurrentDrillingStep();
}

void RemoveWisdomToothWindow::clearCurrentDrillingStep()
{
	if(drillingStepsNode)
		drillingStepsNode->clearCurrentDrillingStep();
}

void RemoveWisdomToothWindow::addDrillingStep()
{
	if(drillingStepsNode)
		drillingStepsNode->addDrillingStep();
}

void RemoveWisdomToothWindow::saveExpertDrillingSteps()
{
	if(drillingStepsNode)
		drillingStepsNode->saveExpertDrillingSteps();
}

void RemoveWisdomToothWindow::enterEditExpertDrillingStepsMode(int enterExpertDrillingMode)
{
	if(drillingStepsNode)
		drillingStepsNode->enterEditExpertDrillingStepsMode(enterExpertDrillingMode);
}


void RemoveWisdomToothWindow::setupStatistics(){

	if(!materialSegmentationModel||!forbiddenSegmentationModel)
		return;

	int nrOfSegments = materialSegmentationModel->segmentNameField.get()->getSize();
	int nrOfForbiddenSegments = forbiddenSegmentationModel->segmentNameField.get()->getSize();

	ui.statisticsTableWidget->setRowCount(nrOfSegments+nrOfForbiddenSegments);
	ui.statisticsTableWidget->setColumnCount(4);

	ui.statisticsTableWidget->setHorizontalHeaderItem(0, new QTableWidgetItem(QString("Material")));
	ui.statisticsTableWidget->setHorizontalHeaderItem(1, new QTableWidgetItem(QString("Total Voxels Removed")));
	ui.statisticsTableWidget->setHorizontalHeaderItem(2, new QTableWidgetItem(QString("Expert Voxels Removed")));
	ui.statisticsTableWidget->setHorizontalHeaderItem(3, new QTableWidgetItem(QString("Fraction vs Expert")));

	for (int i = 0;i < nrOfSegments-1;i++) //not interested in air
	{

		ui.statisticsTableWidget->setItem( i, 0, new QTableWidgetItem(QString((
				materialSegmentationModel->segmentNameField.get()->getValue()[i+1]).c_str())));
		ui.statisticsTableWidget->setItem( i, 1, new QTableWidgetItem(QString("0")));
		ui.statisticsTableWidget->setItem( i, 2, new QTableWidgetItem(QString("0")));
		ui.statisticsTableWidget->setItem( i, 3, new QTableWidgetItem(QString("0")));
	}

	ui.statisticsTableWidget->setItem( nrOfSegments-1, 0, new QTableWidgetItem(QString("Total")));
	ui.statisticsTableWidget->setItem( nrOfSegments-1, 1, new QTableWidgetItem(QString("0")));
	ui.statisticsTableWidget->setItem( nrOfSegments-1, 2, new QTableWidgetItem(QString("0")));
	ui.statisticsTableWidget->setItem( nrOfSegments-1, 3, new QTableWidgetItem(QString("0")));

	QFont f = ui.statisticsTableWidget->item( nrOfSegments-1, 0)->font();
	f.setBold(true);

	ui.statisticsTableWidget->item( nrOfSegments-1, 0)->setFont(f);
	ui.statisticsTableWidget->item( nrOfSegments-1, 1)->setFont(f);
	ui.statisticsTableWidget->item( nrOfSegments-1, 2)->setFont(f);
	ui.statisticsTableWidget->item( nrOfSegments-1, 3)->setFont(f);


	for (int i = 0;i < nrOfForbiddenSegments;i++)
	{
		ui.statisticsTableWidget->setItem( i+nrOfSegments, 0, new QTableWidgetItem(QString((forbiddenSegmentationModel->segmentNameField.get()->getValue()[i]).c_str())));
		ui.statisticsTableWidget->setItem( i+nrOfSegments, 1, new QTableWidgetItem(QString("0")));

		ui.statisticsTableWidget->item( i+nrOfSegments, 0)->setTextColor(QColor(255,85,127,255));
		ui.statisticsTableWidget->item( i+nrOfSegments, 1)->setTextColor(QColor(255,85,127,255));
	}
}


void RemoveWisdomToothWindow::setupObjectives()
{
	if(!stateMachine||!forbiddenSegmentationModel)
		return;

	int nrOfStates = stateMachine->stateObjectivesField.get()->getSize();

	ui.stateTableWidget->setRowCount(nrOfStates);
	ui.stateTableWidget->setColumnCount(1);
	ui.stateTableWidget->setHorizontalHeaderItem(0, new QTableWidgetItem(QString("Objectives")));

	for (int i = 0;i < nrOfStates;i++)
	{
		ui.stateTableWidget->setItem( i, 0, new QTableWidgetItem(QString((
				stateMachine->stateObjectivesField.get()->getValue()[i]).c_str())));
	}
	ui.stateTableWidget->resizeRowsToContents();

}



void RemoveWisdomToothWindow::toggleShowHideHead()
{
	if(ui.headButton->isChecked())
	{
		showHead->whichChoice.get()->setValue(0);
		showRoom->whichChoice.get()->setValue(0);
		ui.rotationPlusY_2->setEnabled(false);
		ui.rotationPlusZ_2->setEnabled(false);
		ui.rotationMinusY_2->setEnabled(false);
		ui.rotationMinusZ_2->setEnabled(false);
		//Reset rotation of teeth.
		Rotation temp(orgRot,orgAngle);
		SFRotation* teethRot = teeth->rotation.get();
		teethRot->setValue(temp);
	}
	else
	{
		showHead->whichChoice.get()->setValue(1);
		showRoom->whichChoice.get()->setValue(1);
		ui.rotationPlusY_2->setEnabled(true);
		ui.rotationPlusZ_2->setEnabled(true);
		ui.rotationMinusY_2->setEnabled(true);
		ui.rotationMinusZ_2->setEnabled(true);
	}
}
void RemoveWisdomToothWindow::toggleShowHideTeeth()
{
	if(ui.teethButton->isChecked())
	{
		showTeeth->whichChoice.get()->setValue(0);
	}
	else{
		showTeeth->whichChoice.get()->setValue(1);
	}
}
void RemoveWisdomToothWindow::toggleShowHideTool()
{
	if(ui.toolButton->isChecked())
	{
		if(ui.selectDrillButton->isChecked())
			selectDrill();
		if(ui.selectElevatorButton->isChecked())
			selectElevator();
	}
	else{
		if(deviceShape!=0 && drillable!=0 && vibrations!=0)
		{
			deviceShape->whichChoice.get()->setValue(2);
			drillable->whichChoice.get()->setValue(1);
			vibrations->whichChoice.get()->setValue(0);
		}
	}
}


void RemoveWisdomToothWindow::highlightUser()
{
	if(highlightNode)
		highlightNode->highlightUser(ui.highlightUserButton->isChecked());
}

void RemoveWisdomToothWindow::highlightExpert()
{
	if(highlightNode)
		highlightNode->highlightExpert(ui.highlightExpertButton->isChecked());
}
void RemoveWisdomToothWindow::highlightForbidden()
{
	if(highlightNode)
		highlightNode->highlightForbidden(ui.highlightForbiddenButton->isChecked());
}

void RemoveWisdomToothWindow::selectDrill()
{
	if(ui.selectDrillButton->isChecked()&&ui.toolButton->isChecked())
	{
		if(drillable!=0 && deviceShape!=0)
		{
			drillable->whichChoice.get()->setValue(0);
			deviceShape->whichChoice.get()->setValue(0);
			aDrillableNode->drillSpeed.get()->setValue(0); //Solved problem that drillspeed is still "on" (>0) if drill is on and is toggled off and than back but the drillimage is "off"
		}
	}
}

void RemoveWisdomToothWindow::selectElevator()
{
	if(ui.selectElevatorButton->isChecked()&&ui.toolButton->isChecked())
	{
	    drillable->whichChoice.get()->setValue(0);
		if(deviceShape!=0 && vibrations!=0)
		{
			deviceShape->whichChoice.get()->setValue(1);
			vibrations->whichChoice.get()->setValue(0);
			aDrillableNode->drillSpeed.get()->setValue(0);
		}
	}
}

void RemoveWisdomToothWindow::disablePlayback()
{
	ui.rw_2->setEnabled(false);
	ui.playstop_2->setEnabled(false);
	ui.ffw_2->setEnabled(false);
	ui.reset_2->setEnabled(false);
	ui.playbacktime_2->setEnabled(false);
	ui.playbacklabel_2->setEnabled(false);
	ui.toolButton->setChecked(true);
	toggleShowHideTool();
	ui.toolButton->setEnabled(true);
}

void RemoveWisdomToothWindow::enablePlayback()
{
	ui.rw_2->setEnabled(true);
	ui.playstop_2->setEnabled(true);
	ui.ffw_2->setEnabled(true);
	ui.reset_2->setEnabled(true);
	ui.playbacktime_2->setEnabled(true);
	ui.playbacklabel_2->setEnabled(true);
	ui.toolButton->setChecked(false);
	toggleShowHideTool();
	ui.toolButton->setEnabled(false);
}

