#ifndef REMOVEWISDOMTOOTHWINDOW_H
#define REMOVEWISDOMTOOTHWINDOW_H

#include <QtGui/QMainWindow>
#include "ui_RemoveWisdomToothWindow.h"
#include "ApplicationNode.h"

#include "../src/VolumeModel.h"
#include "../src/MaterialSegmentationModel.h"
#include "../src/ADrillableNode.h"
#include "../src/DrillingStepsNode.h"
#include "../src/HighlightNode.h"
#include "../src/DefaultValues.h"
#include "../src/SaveVolume.h"
#include "../src/StateMachine.h"
#include "../src/ForbiddenSegmentationModel.h"
#include "../src/AVolumePlaybackNode.h"

#include <H3D/Transform.h>
#include <H3D/PhantomDevice.h>
#include <H3D/Switch.h>

#include "Updater_QLabel_SFFloat.h"
#include "Updater_QDial_SFInt32.h"

// old code
#include "Updater_QSlider_MaterialHardness.h"
#include "Updater_QSlider_SFFloat.h"
#include "Updater_QButton_SaveAndExit.h"
#include "Updater_QButton_NextState.h"
#include "Updater_QButton_RemovePartitionByName.h"

#include "Updater_QLabel_SFInt32.h"
#include "Updater_QLabel_SFVec3f.h"
#include "Updater_QLabel_MFInt32.h"

#include "Updater_QLabel_SFTranslation.h"
#include "Updater_QLabel_SFRotation.h"

#include "Updater_QRadioButton_HighlightType.h"

#include "Updater_QButton_Playback.h"
#include "Updater_QButton_SFBoolPB.h"
#include "Updater_QButton_SFFloat.h"
#include "Updater_QButton_SFInt32_Drill_Elevator.h"

#include "Updater_QProgressBar_PlaybackProgressBar.h"

#include "Updater_QTableWidget.h"


using namespace H3D;

namespace FS
{

class ApplicationNode;
class Updater_QButton_SaveAndExit;

class RemoveWisdomToothWindow : public QMainWindow
{
    Q_OBJECT

public:
	RemoveWisdomToothWindow(ApplicationNode *applicationNode, bool playback, QWidget* parent = 0);
    ~RemoveWisdomToothWindow();

    void initialize(bool playback);
    void setupStatistics();
    void setupObjectives();
    void DestroyRemoveWisdomToothWindow();

private slots:

	void decrementCurrentDrillingStep();
	void incrementCurrentDrillingStep();
	void clearCurrentDrillingStep();
	void addDrillingStep();
	void saveExpertDrillingSteps();
	void enterEditExpertDrillingStepsMode(int enterExpertDrillingMode);
	void toggleShowHideHead();
	void toggleShowHideTool();
	void toggleShowHideTeeth();
	void highlightUser();
	void highlightExpert();
	void highlightForbidden();
	void selectDrill();
	void selectElevator();
	void disablePlayback();
	void enablePlayback();

private:
	Ui::RemoveWisdomToothWindowClass ui;
	ApplicationNode* applicationNode;

	/*This is the QLabels on the statusbar. The route connection is done in ApplicationNode.cpp*/
	auto_ptr<QLabel> frameRateLabel;
	auto_ptr<QLabel> hapticRateLabel;

	// The various nodes of interest to this window
	VolumeModel* volumeModel;
	Transform* room;
	Transform* fakeCamera;
	Transform* patient;
	Transform* teeth;
	Transform* head;
	Transform* gumCut;
	Transform* teethGum;
	Transform* lowerTeeth;
	Transform* upperTeeth;
	Transform* floorOfMouth;
	Transform* tongue;
	Transform* hook;
	ADrillableNode* aDrillableNode;
	PhantomDevice* hapticDeviceNode;
	DrillingStepsNode* drillingStepsNode;
	MaterialSegmentationModel* materialSegmentationModel;
	ForbiddenSegmentationModel* forbiddenSegmentationModel;
	HighlightNode* highlightNode;
	AVolumePlaybackNode* playbackNode;
	DefaultValues* defaultValues;
	SaveVolume* saveVolume;
	Switch* showHead;
	Switch* showRoom;
	Switch* drillable;
	Switch* showTeeth;
	Switch* deviceShape;
	Switch* vibrations;
	StateMachine* stateMachine;
	LoggingNode* loggingNode;
	bool isPlayback;
	H3DFloat orgAngle;
	Vec3f orgRot;

	// routing between fields and widgets

	auto_ptr<Updater_QLabel_SFFloat> updater_label_frameRateField;
	auto_ptr<Updater_QLabel_SFInt32> updater_label_hapticRateLabel;

	// old code
	auto_ptr<Updater_QLabel_SFFloat> updater_label_fractionField;
	auto_ptr<Updater_QSlider_MaterialHardness> updater_slider_materialHardnessField;
	auto_ptr<Updater_QSlider_SFFloat> updater_slider_cutRadiusField;
	auto_ptr<Updater_QSlider_SFFloat> updater_slider_drillSpeedMaxField;

	auto_ptr<Updater_QButton_SaveAndExit> updater_button_saveTimeStampsToFileAndExitField;
	auto_ptr<Updater_QButton_NextState> updater_button_nextStateField;
	auto_ptr<Updater_QButton_RemovePartitionByName> updater_button_removeCrownByNameField;
	auto_ptr<Updater_QButton_RemovePartitionByName> updater_button_removeRootByNameField;
	auto_ptr<Updater_QButton_RemovePartitionByName> updater_button_removeOtherByNameField;

	auto_ptr<Updater_QLabel_SFFloat> updater_label_drillSpeedField;
	auto_ptr<Updater_QLabel_SFVec3f> updater_label_drillPosField;
	auto_ptr<Updater_QLabel_SFVec3f> updater_label_forceField;
	auto_ptr<Updater_QLabel_SFInt32> updater_label_stateField;
	auto_ptr<Updater_QLabel_SFFloat> updater_label_fpsField;
	auto_ptr<Updater_QLabel_SFInt32> updater_label_hpsField;
	auto_ptr<Updater_QLabel_MFInt32> updater_label_forbiddenFields;

	//Model configuration configured via radio buttons and push buttons
	auto_ptr<Updater_QLabel_SFTranslation> updater_label_viewTranslationField;
	auto_ptr<Updater_QLabel_SFRotation> updater_label_viewRotationField;
	auto_ptr<Updater_QLabel_SFTranslation> updater_label_patTranslationField;
	auto_ptr<Updater_QLabel_SFRotation> updater_label_patRotationField;
	auto_ptr<Updater_QLabel_SFTranslation> updater_label_headTranslationField;
	auto_ptr<Updater_QLabel_SFRotation> updater_label_headRotationField;
	auto_ptr<Updater_QLabel_SFTranslation> updater_label_teethTranslationField;
	auto_ptr<Updater_QLabel_SFRotation> updater_label_teethRotationField;
	auto_ptr<Updater_QLabel_SFTranslation> updater_label_gumCutTranslationField;
	auto_ptr<Updater_QLabel_SFRotation> updater_label_gumCutRotationField;
	auto_ptr<Updater_QLabel_SFTranslation> updater_label_lowerTeethTranslationField;
	auto_ptr<Updater_QLabel_SFRotation> updater_label_lowerTeethRotationField;
	auto_ptr<Updater_QLabel_SFTranslation> updater_label_upperTeethTranslationField;
	auto_ptr<Updater_QLabel_SFRotation> updater_label_upperTeethRotationField;
	auto_ptr<Updater_QLabel_SFTranslation> updater_label_teethGumTranslationField;
	auto_ptr<Updater_QLabel_SFRotation> updater_label_teethGumRotationField;
	auto_ptr<Updater_QLabel_SFTranslation> updater_label_tongueTranslationField;
	auto_ptr<Updater_QLabel_SFRotation> updater_label_tongueRotationField;
	auto_ptr<Updater_QLabel_SFTranslation> updater_label_hookTranslationField;
	auto_ptr<Updater_QLabel_SFRotation> updater_label_hookRotationField;
	auto_ptr<Updater_QLabel_SFTranslation> updater_label_floorOfMouthTranslationField;
	auto_ptr<Updater_QLabel_SFRotation> updater_label_floorOfMouthRotationField;

	auto_ptr<Updater_QSlider_SFFloat> updater_slider_translationZoomFactorField;
	auto_ptr<Updater_QSlider_SFFloat> updater_slider_rotationZoomFactorField;

	auto_ptr<Updater_QLabel_SFFloat> updater_label_playbacktime;
	auto_ptr<Updater_QButton_Playback> updater_button_playstop;
	auto_ptr<Updater_QButton_SFBoolPB> updater_button_reset;
	auto_ptr<Updater_QButton_SFFloat> updater_button_fastforward;
	auto_ptr<Updater_QButton_SFFloat> updater_button_rewind;
	auto_ptr<Updater_QProgressBar_PlaybackProgressBar> updater_progressbar_playbackProgressBar;

	auto_ptr<Updater_QLabel_SFFloat> updater_label_playbacktime_2;
	auto_ptr<Updater_QButton_Playback> updater_button_playstop_2;
	auto_ptr<Updater_QButton_SFBoolPB> updater_button_reset_2;
	auto_ptr<Updater_QButton_SFFloat> updater_button_fastforward_2;
	auto_ptr<Updater_QButton_SFFloat> updater_button_rewind_2;
	auto_ptr<Updater_QProgressBar_PlaybackProgressBar> updater_progressbar_playbackProgressBar_2;
	auto_ptr<Updater_QButton_SFInt32_Drill_Elevator> update_button_drill;
	auto_ptr<Updater_QButton_SFInt32_Drill_Elevator> update_button_elevator;

	auto_ptr<Updater_QLabel_SFInt32> updater_label_stateField_2;
	auto_ptr<Updater_QLabel_SFFloat> updater_label_fpsField_2;
	auto_ptr<Updater_QLabel_SFInt32> updater_label_hpsField_2;
	auto_ptr<Updater_QLabel_SFRotation> updater_label_teethRotationField_2;

	auto_ptr<Updater_QTableWidget> updater_tableWidget_statisticsField;
};

}

#endif // REMOVEWISDOMTOOTHWINDOW_H
