#ifndef SELECTCASEWINDOW_H
#define SELECTCASEWINDOW_H

#include <QtGui/QMainWindow>
#include <QFileInfo>
#include "ui_SelectCaseWindow.h"
#include "ApplicationNode.h"
#include "QDebugStream.h"

#include <H3D/X3DSAX2Handlers.h>

using namespace std;

namespace FS
{

class ApplicationNode;

/**
 * Allows the Student to select which operation they wish to execute
 */
class SelectCaseWindow : public QMainWindow
{
    Q_OBJECT

public:
    SelectCaseWindow(ApplicationNode *applicationNode, QWidget *parent = 0);
    ~SelectCaseWindow();

    void initialize();
    void showFullScreenAndUpdate();

public slots:
	/// locates an x3d file to execute
	void browseToX3dFile();

	void browseToCaseFile();

	/// Launches a x3d file as selected via the browse button
	void launchX3dFile();

	/// Closes a previously loaded x3d file
	void closeX3dFile();

	/// Set if the GlWindow is to be mirrored or not
	void setIsMirrored(int state);

	/// Set if the GlWindow is to be in stereo mode or not
	void setIsStereo(int state);

	void launchCaseWithPlayback();
	void launchCaseWithoutPlayback();

	void updateCaseListTimestampsWidget();
	void updateAnamnesTextWidget();
	void updateObjectivesTextWidget();
	void getCurrentTimestampForPlayback();
	void updateImageViewer();
	void setCloseX3DFileEnable(bool enable);



private:
    Ui::SelectCaseWindowClass ui;
    ApplicationNode* applicationNode;
    auto_ptr<QDebugStream> standardOutputToTextEdit;
    auto_ptr<QDebugStream> standardErrorToTextEdit;

    void launchCase(bool isPlayback);
};

}

#endif // SELECTCASEWINDOW_H
