/**
 * Updater_QLabel_MFFloat.h
 *
 *  Created on: Mar 25, 2009
 *      Author: barrygr
 */

#ifndef UPDATER_QLABEL_MFINT32_H_
#define UPDATER_QLABEL_MFINT32_H_

#include <QtGui>
#include <H3D/X3D.h>
#include <H3D/MFInt32.h>
#include <iostream>
#include <stdexcept>

using namespace std;
using namespace H3D;

namespace FS
{

/**
 * This class connects a set of QLabels with a MFInt32 field. When a value in the field updates, this update
 * is reflected in the corresponding label.
 */
class Updater_QLabel_MFInt32 : public AutoUpdate< MFInt32 >
{
public:
	/**
	 * Constructs an auto update MFInt32 field, which is routed to another MFInt32 field, namely fieldToRouteTo.
	 * When this other field updates, this value is reflected in the corresponding label.
	 * \param labels The list of labels to update. If the list of label pointers contains a null value, then no
	 * label is expected to be updated for the corresponding value in fieldToRouteTo. For example, if fieldToRouteTo
	 * contains two values and the first value in labels points to a null and the second to a label, then updates in the
	 * first value in fieldToRouteTo will be ignored, but updates in the second value will be seen in the label pointed to
	 * by labels
	 * \param
	 */
	Updater_QLabel_MFInt32(vector<QLabel*> labels, MFInt32* fieldToRouteTo);
	virtual ~Updater_QLabel_MFInt32();

	/**
	 * Called each time a value in fieldToRouteTo is changed
	 */
	virtual void update();

	/**
	 * Exception class for Updater_QLabel_MFInt32
	 */
	class Updater_QLabel_MFInt32Exception: public runtime_error
	{
	public:

		/**
		 * Constructor
		 */
		Updater_QLabel_MFInt32Exception(const string& what) :
			runtime_error("Exception within Updater_QLabel_MFInt32: " + what)
			{
			}
	};

private:
	vector<QLabel*> labels;
	const MFInt32* fieldToRouteTo;
};
}

#endif /* UPDATER_QLABEL_MFINT32_H_ */
