/*
 * Updater_QSLider_MaterialHardness.cpp
 *
 *  Created on: April 2, 2009
 *      Author: jockeka
 */

#include "Updater_QSlider_MaterialHardness.h"

using namespace FS;

Updater_QSlider_MaterialHardness::Updater_QSlider_MaterialHardness(Ui::RemoveWisdomToothWindowClass ui, MaterialSegmentationModel* model, MFFloat* fieldToRouteTo)
{
	/*Initialization of variables needs to be done before fieldToRouteTo->route(this) since
	 * since that will trigger a call to the update() function.
	 */
	ui.currentMaterialName->setText("");
	currentSegId = 1;

	this->model = model;
	this->ui = ui;
	this->fieldToRouteTo = fieldToRouteTo;
	this->setValue(fieldToRouteTo->getValue());

	this->route(fieldToRouteTo);
	fieldToRouteTo->route(this);

	//Initialize GUI to first segment in list
	ui.currentMaterialName->setText(QString::fromStdString(model->getName(currentSegId)));
	SFFloat currentHardness;
	currentHardness.setValue(model->getHardness(currentSegId));
	SFFloat sliderPosition;
	sliderPosition.setValue((currentHardness.getValue()/MAX_HARDNESS)*ui.materialHardnessSlider->maximum());
	//Slider position
	ui.materialHardnessSlider->setValue((int)(sliderPosition.getValue()+0.5));
	std::string s = currentHardness.getValueAsString();
	//Label value
	ui.materialHardness->setText(s.c_str());

	QObject::connect(ui.materialHardnessSlider, SIGNAL(valueChanged (int)), this, SLOT(setHardness(int)));
	QObject::connect(ui.nextMaterialButton, SIGNAL(clicked()), this, SLOT(nextMaterial()));
}

Updater_QSlider_MaterialHardness::~Updater_QSlider_MaterialHardness()
{
	cout<<"Destroying Updater_QSlider_MaterialHardness"<<endl;
	QObject::disconnect(ui.materialHardnessSlider, SIGNAL(valueChanged (int)));
	QObject::disconnect(ui.nextMaterialButton, SIGNAL(clicked()));
	this->unrouteAll();
}


void Updater_QSlider_MaterialHardness::update()
{
	//MFFloat* segmentHardnessField = static_cast< MFFloat * >(routes_in[0])->getValue();
	//slider->setValue(value.getValue()/MAX_HARDNESS/INCREMENT_HARDNESS_BY);


	/* Update Hardness label. The slider is already "set" by user-interaction,
	 * the model->setHardness() function call will trigger this update() function by routing.
	 */
	SFFloat currentHardness;
	currentHardness.setValue(model->getHardness(currentSegId));
	std::string s = currentHardness.getValueAsString();
	ui.materialHardness->setText(s.c_str());

}

void Updater_QSlider_MaterialHardness::setHardness(int value)
{
	float newHardness;
	char str[10];
	newHardness = ((float)value/ui.materialHardnessSlider->maximum())*MAX_HARDNESS;
	sprintf(str, "%.2f",newHardness); //Round value to 2 decimals.
	model->setHardness(currentSegId,atof(str));

}

void Updater_QSlider_MaterialHardness::nextMaterial()
{
	int nbrOfSegments = model->getNoOfSegments();
	currentSegId++;
	if (currentSegId>=nbrOfSegments){
		currentSegId=1;//0 is Air and we don't want to change hardness for it.
	}
	cout<<"The current seg id is "<<currentSegId<<endl;
	ui.currentMaterialName->setText(QString::fromStdString(model->getName(currentSegId)));

	//Also update slider and label hardness values in GUI
	SFFloat currentHardness;
	currentHardness.setValue(model->getHardness(currentSegId));
	SFFloat sliderPosition;
	sliderPosition.setValue((currentHardness.getValue()/MAX_HARDNESS)*ui.materialHardnessSlider->maximum());
	//Slider position
	ui.materialHardnessSlider->setValue((int)(sliderPosition.getValue()+0.5));
	std::string s = currentHardness.getValueAsString();
	//Label value
	ui.materialHardness->setText(s.c_str());
}
