/**
 * ApplicationNode.cpp
 *
 *  Created on: Feb 9, 2009
 *      Author: barrygr
 */

#include "ApplicationNode.h"

using namespace FS;

/// Add this node to the H3DNodeDatabase system.
H3D::H3DNodeDatabase ApplicationNode::database( "ApplicationNode",
		&(newInstance<ApplicationNode>),
		typeid( ApplicationNode ),
		&X3DChildNode::database );

namespace ApplicationNodeInternals
{
}

ApplicationNode::ApplicationNode(int argc=1, char *argv[]=0)
	: QApplication(argc, argv), quitApplicationBeforeExec(false)
{
	type_name = "ApplicationNode";
	database.initFields( this );

	#ifdef H3DAPI_LIB
	  initializeH3D();
	#endif

	#ifdef HAVE_PYTHON
	  PythonScript::setargv( argc, argv );
	#endif

	// H3D root, XML file and setup file
    vector<string> xml_files;
    char *r = getenv( "H3D_ROOT" );

    string h3d_root = r ? r : "";
    INIFile ini_file( h3d_root + "/settings/h3dload.ini" );

    // Console settings
    bool console_show_time = GET_BOOL("console", "show_time", false);
    bool console_show_level = GET_BOOL("console", "show_level", false);
    int console_output_level = GET_INT("console", "output_level", 3 );

    ostream *console_ostream = 0;
    string ostream_str = GET4("H3D_CONSOLE_OSTREAM",
    	"console","ostream",(string)"cerr");
    if ( ostream_str == "cerr" )
    	console_ostream = &cerr;
    else if( ostream_str == "cout" )
    	console_ostream = &cout;
    else
    	Console(4) << "Invalid value \"" <<  ostream_str
			<< "\" on ostream variable. Must be cerr or cout."<< endl;

    Console.setShowTime(console_show_time);
    Console.setShowLevel(console_show_level);
    Console.setOutputLevel(console_output_level);
    Console.setOutputStream(*console_ostream);

    Console(3) << "Console created" << endl;

    // Graphics, devices, models and such
    string settings_path =
      GET_ENV_INI_DEFAULT( "H3D_DISPLAY",
      h3d_root + "/settings/display/",
      "display","type",
      h3d_root + "/settings/common/" );

    string common_path =  h3d_root + "/settings/common/";

    string deviceinfo_file =
      GetEnvIniDefaultFile( ini_file, "H3D_DEFAULT_DEVICEINFO",
      settings_path + "/device/",
      common_path + "/device/",
      "haptics device","device" );

    string stylus_file =
      GetEnvIniDefaultFile( ini_file, "H3D_STYLUS",
      common_path + "/stylus/",
      common_path + "/stylus/",
      "haptics device","stylus" );

    string viewpoint_file =
      GetEnvIniDefaultFile( ini_file, "H3D_DEFAULT_VIEWPOINT",
      settings_path + "/viewpoint/",
      common_path + "/viewpoint/",
      "graphical", "viewpoint" );

    string render_mode = GET4( "H3D_RENDERMODE",
      "graphical", "rendermode",
      (string)"MONO" );

    int width = GET_INT("graphical", "width", 640 );
    int height = GET_INT("graphical", "height", 480 );

    bool fullscreen    = GET_BOOL("graphical", "fullscreen", false);
    if( char *buffer = getenv("H3D_FULLSCREEN") )
    {
      if (strcmp( buffer, "TRUE" ) == 0 )
        fullscreen = true;
      else if (strcmp( buffer, "FALSE" ) == 0 )
        fullscreen = false;
      else
        Console(4) << "Invalid value \"" << buffer
        << "\" on environment "
        << "variable H3D_FULLSCREEN. Must be TRUE or FALSE. "
        << endl;
    }

    bool mirrored      = GET_BOOL("graphical", "mirrored", false);
    if( char *buffer = getenv("H3D_MIRRORED") ) {
      if (strcmp( buffer, "TRUE" ) == 0 )
        mirrored = true;
      else if (strcmp( buffer, "FALSE" ) == 0 )
        mirrored = false;
      else
        Console(4) << "Invalid value \"" << buffer
        << "\" on environment "
        << "variable H3D_MIRRORED. Must be TRUE or FALSE. "<< endl;
    }

    bool use_space_mouse = false;

    Console(3) << "Init Params Read" << endl;

    // Command line arguments ---

    for( int i = 1 ; i < argc ; i++ )
    {
    	if( argv[i][0] != '-' )
    	{
    		xml_files.push_back( string(argv[i]) );
    		continue;
    	}

    	switch(argv[i][1])
    	{
			// Long arguments ---

			case '-':

				if( !strcmp(argv[i]+2,"help") )
				{
					std::cout << GetHelpMessage(argv[0]) << endl;
					quitApplicationBeforeExec = true;
					return;
				}

				else if( !strcmp(argv[i]+2,"mirror") )
					mirrored = true;
				else if( !strcmp(argv[i]+2,"no-mirror") )
					mirrored = false;
				else if( !strcmp(argv[i]+2,"fullscreen") )
					fullscreen = true;
				else if( !strcmp(argv[i]+2,"no-fullscreen") )
					fullscreen = false;
				else if( !strncmp(argv[i]+2,"screen=", strlen("screen=")) )
				{
					height = atoi( strstr(argv[i],"x")+1 );
					*strstr(argv[i],"x") = '\0';
					width = atoi( strstr(argv[i],"=")+1 );
				}
				else if( !strncmp(argv[i]+2,"deviceinfo=", strlen("deviceinfo=")) )
					deviceinfo_file = strstr(argv[i],"=")+1;
				else if( !strncmp(argv[i]+2,"stylus=", strlen("stylus=")) )
					stylus_file = strstr(argv[i],"=")+1;
				else if( !strncmp(argv[i]+2,"viewpoint=", strlen("viewpoint=")) )
					viewpoint_file = strstr(argv[i],"=")+1;
				else if( !strncmp(argv[i]+2,"rendermode=", strlen("rendermode=")) )
					render_mode = strstr(argv[i],"=")+1;
				else if( !strcmp(argv[i]+2,"spacemouse") )
					use_space_mouse = true;
				else
					Console(4) << "Unknown argument " << "'" << argv[i] << "'" << endl;
				break;

				// Short arguments ---

				case 'm':
					mirrored = true;
					break;

				case 'M':
					mirrored = false;
					break;

				case 'f':
					fullscreen = true;
					break;

				case 'F':
					fullscreen = false;
					break;

				case 's':
					use_space_mouse = true;
					break;

				default:
					Console(4) << "Unknown argument " << "'" << argv[i] << "'" << endl;
		}
	}

	if (!xml_files.size())
	{
		std::cout << GetHelpMessage(argv[0]) << endl;
		quitApplicationBeforeExec = true;
		return;
	}

	Console(3) << "Loading DeviceInfo 1" << endl;

    // Loading X3D file and setting up VR environment ---
	 try
	 {

    AutoRef< Node > device_info;
    AutoRef< Node > viewpoint;

    Console(3) << "Loading DeviceInfo 2" << endl;

    if( deviceinfo_file.size() )
    {
    	try
    	{
    		device_info = X3D::createX3DNodeFromURL( deviceinfo_file );
	    }
    	catch( const Exception::H3DException &e )
    	{
	        Console(3) << "Warning: Could not create default DeviceInfo node "
	          << "from file \"" << deviceinfo_file << "\": "
	          << e << endl;
	    }
	 }

     Console(3) << "Loading Stylus" << endl;

	 DeviceInfo *di = DeviceInfo::getActive();
	 if( di && stylus_file.size() )
	 {
		 AutoRef< Node > default_stylus;
	     try
	     {
	        default_stylus = X3D::createX3DNodeFromURL( stylus_file );
	     }
	     catch( const Exception::H3DException &e )
	     {
	        Console( 4 ) << "Warning: Could not create default stylus "
	          << "from file \"" << stylus_file << "\": "
	          << e << endl;
	     }

	     for( DeviceInfo::MFDevice::const_iterator i = di->device->begin();
	        i != di->device->end(); i++ )
	     {
	          H3DHapticsDevice *d = static_cast< H3DHapticsDevice * >(*i);
	          if( !d->stylus->getValue() )
	            d->stylus->setValue( default_stylus );
	     }
	 }

	 X3D::DEFNodes defNodes;
	 group.reset( new Group );

	 for( vector<string>::iterator file = xml_files.begin();
		 file != xml_files.end() ; file++ )
	 {
		 Console(3) << "Loading " << *file << endl;
		 if ( file->size() > 4 && file->find( ".wrl", file->size()-5 ) != string::npos )
			 group->children->push_back( X3D::createVRMLFromURL( *file, &defNodes ) );
		 else
			 group->children->push_back( X3D::createX3DFromURL( *file, &defNodes ) );
		 Console(3) << "File Loaded" << endl;
	 }

#ifndef MACOSX
    if( use_space_mouse )
    	if( !H3DNavigation::isEnabled( H3DNavigation::SWS ) )
    		H3DNavigation::enableDevice( H3DNavigation::SWS );
#endif

     Console(3) << "Creating ViewPoint" << endl;

     // create a Viewpoint if it does not exist.
	 if( !X3DViewpointNode::getActive() && viewpoint_file.size() )
	 {
	      try
	      {
	        viewpoint = X3D::createX3DNodeFromURL( viewpoint_file );
	      }
	      catch( const Exception::H3DException &e )
	      {
	        Console(3) << "Warning: Could not create default Viewpoint node "
	          << "from file \"" << viewpoint_file << "\": "
	          << e << endl;
	      }
	 }

	 if( X3DBindableNode::getStack( "DeviceInfo" ).size() > 1 )
		 device_info.reset( NULL );

	 defNodes.clear();

	// handle escape
	quitApplicationField.reset(new QuitApplicationField);
	keySensor.reset(new KeySensor);
	keySensor->keyPress->route( quitApplicationField );

	// create scene object
	scene.reset(new Scene);

	// create gl window object

	Console(3) << "Setting up glWindow" << endl;

	cout << "rendermode " << render_mode<<endl;
	if(render_mode=="MONO")
	  glWindow.reset	(new GlWindowNode(false));
	else
          glWindow.reset	(new GlWindowNode(true));

	glWindow->fullscreen->setValue( fullscreen );
	glWindow->mirrored->setValue( mirrored );
	glWindow->renderMode->setValue( render_mode  );
	glWindow->SetWidth(width);
	glWindow->SetHeight(height);
	scene->window->push_back( glWindow.get() );
	scene->sceneRoot->setValue( group.get() );
	glWindow->setLastRenderChild( group.get() );

	Console(3) << "Setting up glWindow timer" << endl;

	// update the scene graph in a periodic fashion
	timer.reset(new QTimer(this));
	QObject::connect(timer.get(), SIGNAL(timeout()), glWindow.get(), SLOT(updateScene()));
	timer->start();

	Console(3) << "glWindow timer started" << endl;

	// display gl on the second screen
	glWindow->setGeometry(QApplication::desktop()->availableGeometry(1));
	glWindow->show();

	Console(3) << "glWindow displayed" << endl;


   }
   catch (const QuitAPIException &)
   {
	   Console(4) << "QuitAPIException" << endl;
	   quitApplicationBeforeExec = true;
	   return;
   }
   catch (const Exception::H3DException &e)
   {
	 Console(4) << "Exception::H3DException:" << e << endl;
     quitApplicationBeforeExec = true;
	 return;
   }
   catch (const Exception::H3DException &e)
   {
   	 Console(4) << "Exception::H3DException:" << e << endl;
        quitApplicationBeforeExec = true;
   	 return;
   }
   catch(...)
   {
	   Console(4) << "Exception ..." << endl;
	   quitApplicationBeforeExec = true;
	   return;
   }

	// display gui on the first screen
	//guiWindow.setGeometry(QApplication::desktop()->availableGeometry(0));
	//guiWindow.showMaximized();
}

string ApplicationNode::GetEnvIniDefaultFile( INIFile &ini_file,
		const string &ENV,
        const string &DISPLAY_PATH,
        const string &COMMON_PATH,
        const string &GROUP,
        const string &VAR )
{
	char *env = getenv(ENV.c_str());
    if( env )
    	return env;

    if( ini_file.hasOption(GROUP,VAR) )
    {
    	string option = ini_file.get( GROUP, VAR );

    	ifstream inp( (DISPLAY_PATH + option).c_str() );
        inp.close();
        if(!inp.fail())
        	return DISPLAY_PATH + option;

        inp.clear();
        inp.open( (COMMON_PATH + option).c_str() );
        inp.close();
        if(!inp.fail())
        	return COMMON_PATH + option;
    }
    return "";
}

string ApplicationNode::GetHelpMessage(string programName)
{
	  string help_message = "\n";
	  help_message += "Usage: " + programName + " [Options] <X3D file>\n";
	  help_message += "\n";
	  help_message += "Options:\n";
	  help_message += "\n";
	  help_message += " -m --mirror            Flip screen upside down\n";
	  help_message += " -M --no-mirror         Surpress flipping screen\n";
	  help_message += " -f --fullscreen        Open in fullscreen mode\n";
	  help_message += " -F --no-fullscreen     Surpress fullscreen mode\n";
	  help_message += "    --screen=<W>x<H>    Set width and height for window\n";
	  help_message += "\n";
	  help_message += "    --deviceinfo=<file> Use <file> as device info\n";
	  help_message += "    --stylus=<file>     Use <file> as stylus model\n";
	  help_message += "    --viewpoint=<file>  Use <file> as viewpoint\n";
	  help_message += "    --rendermode=<mode> Use render mode <mode>\n";
	  help_message += " -s --spacemouse        Use 3DConnexion space mouse to\n";
	  help_message += "                        navigate scene.\n";
	  help_message += "\n";
	  help_message += " -h --help              This help message\n";
	  help_message += "\n";

	  return help_message;
}

