#include "GlWindowNode.h"

using namespace H3D;

// Add this node to the H3DNodeDatabase system.
H3DNodeDatabase GlWindowNode::database( "GlWindowNode",
		&(newInstance<GlWindowNode>),
        typeid( GlWindowNode ),
        &(H3DWindowNode::database) );

GlWindowNode::GlWindowNode(bool isStereo,
		Inst< SFInt32     > _width,
		Inst< SFInt32     > _height,
        Inst< SFBool      > _fullscreen,
        Inst< SFBool      > _mirrored,
        Inst< RenderMode  > _renderMode,
        Inst< SFViewpoint > _viewpoint,
        Inst< SFTime      > _time
        ) :
        	H3DWindowNode( _width, _height, _fullscreen,
        			_mirrored, _renderMode, _viewpoint, _time )
{
  type_name = "GlWindowNode";
  database.initFields( this );

  this->setMouseTracking(true);

  if(isStereo)
  { 
    cout << "stereo " << isStereo<<endl;
    QGLFormat glFormat( QGL::DoubleBuffer  | QGL::AlphaChannel | QGL::DepthBuffer | QGL::Rgba | QGL::StencilBuffer | QGL::StereoBuffers );
    setFormat(glFormat);
  }
  else
    cout << "mono " << isStereo<<endl;
}

GlWindowNode::~GlWindowNode()
{
}

void GlWindowNode::initWindow()
{
  glInit();
  QGLWidget::resize( H3DWindowNode::width->getValue(), H3DWindowNode::height->getValue() );
  glClearColor( 0, 0, 0, 1 );
  glColorMask(GL_TRUE, GL_TRUE, GL_TRUE, GL_TRUE);
  glClear( GL_COLOR_BUFFER_BIT );
}

void GlWindowNode::setFullscreen( bool fullscreen )
{
  if( fullscreen )
	  showFullScreen();
  else
	  showNormal();
}

void GlWindowNode::swapBuffers()
{
  QGLWidget::swapBuffers();
}

void GlWindowNode::makeWindowActive()
{
  QGLWidget::makeCurrent();
}

void GlWindowNode::initializeGL()
{
  QGLWidget::initializeGL();
}
void GlWindowNode::paintGL()
{
  display();
}

void GlWindowNode::resizeGL(int width, int height)
{
  QGLWidget::resizeGL( width, height );
  reshape( width, height );
  glClear( GL_COLOR_BUFFER_BIT );
}

void GlWindowNode::keyPressEvent(QKeyEvent *event)
{
	QByteArray byteArray = event->text().toAscii();

	for (int i = 0; i < byteArray.size(); i++)
		this->onKeyDown(byteArray.at(i),false);
}

void GlWindowNode::mouseMoveEvent(QMouseEvent *event)
{
	Qt::MouseButtons mouseButtons = event->buttons();
	onMouseMotionAction( event->x(), event->y() );
}

void GlWindowNode::mousePressEvent(QMouseEvent *event)
{
	Qt::MouseButtons mouseButtons = event->buttons();

	if(mouseButtons & Qt::LeftButton)
		onMouseButtonAction( H3D::MouseSensor::LEFT_BUTTON, H3D::MouseSensor::DOWN);

	if(mouseButtons & Qt::MidButton)
		onMouseButtonAction( H3D::MouseSensor::MIDDLE_BUTTON, H3D::MouseSensor::DOWN);

	if(mouseButtons & Qt::RightButton)
		onMouseButtonAction( H3D::MouseSensor::RIGHT_BUTTON, H3D::MouseSensor::DOWN);
}

void GlWindowNode::mouseReleaseEvent(QMouseEvent* /*event*/ )
{

	onMouseButtonAction( H3D::MouseSensor::LEFT_BUTTON, H3D::MouseSensor::UP);
	onMouseButtonAction( H3D::MouseSensor::MIDDLE_BUTTON, H3D::MouseSensor::UP);
	onMouseButtonAction( H3D::MouseSensor::RIGHT_BUTTON, H3D::MouseSensor::UP);
}

void GlWindowNode::wheelEvent(QWheelEvent *event)
{
	float bruteDeltaValue =  event->delta()/120.0;

	for(int i = 0; i < abs((long)bruteDeltaValue); i++  )
		if(bruteDeltaValue>0   )
			onMouseWheelAction( MouseSensor::FROM );
		else
			onMouseWheelAction( MouseSensor::TOWARDS );
}

void GlWindowNode::SetWidth(int width)
{
	(H3D::H3DWindowNode::width)->setValue(width);
}

void GlWindowNode::SetHeight(int height)
{
	(H3D::H3DWindowNode::height)->setValue(height);
}

