#ifndef GL_WINDOW_NODE_H
#define GL_WINDOW_NODE_H

#include <QtGui>
#include <QApplication>

#include <fstream>
#include <H3D/X3D.h>
#include <string.h>

#include <iostream>

#include <H3D/VrmlParser.h>
#include <H3D/GLUTWindow.h>
#include <H3D/Group.h>
#include <H3D/Transform.h>
#include <H3D/Scene.h>
#include <H3D/KeySensor.h>
#include <H3D/MouseSensor.h>
#ifndef MACOSX
#include <H3D/SpaceWareSensor.h>
#endif
#include <H3D/DEFNodes.h>
#include <H3D/Viewpoint.h>
#include <H3D/DeviceInfo.h>
#include <H3D/INIFile.h>
#include <H3D/ResourceResolver.h>
#include <H3D/PythonScript.h>
#include <H3D/NavigationInfo.h>
#include <H3D/H3DNavigation.h>

#include <GL/glew.h>
#include <QtOpenGL/QGLWidget>
#include <QtGui>
#include <QtOpenGL/QtOpenGL>

#include <H3D/X3D.h>
#include <H3D/H3DWindowNode.h>
#include <H3D/Scene.h>

#include <math.h>
#include "qth3dload.h"
#include <QtOpenGL/QGLFormat>

using namespace H3D;

namespace H3D {

  /// \ingroup Nodes
  /// \class GLWindow
  /// \brief Standard OpenGL based window node.
  ///
  class GlWindowNode : public QGLWidget, public H3DWindowNode
                               {

  public:
    Q_OBJECT

  public:

    /// Constructor.
    GlWindowNode(bool isStereo  =false,
    		  Inst< SFInt32     > _width      = 0,
              Inst< SFInt32     > _height     = 0,
              Inst< SFBool      > _fullscreen = 0,
              Inst< SFBool      > _mirrored   = 0,
              Inst< RenderMode  > _renderMode = 0,
              Inst< SFViewpoint > _viewpoint  = 0,
              Inst< SFTime      > _time       = 0 );

    /// Destructor.
    ~GlWindowNode();

    virtual void swapBuffers();

    virtual void initWindow();

    virtual void initWindowHandler() { }

    virtual void setFullscreen( bool fullscreen );

    virtual void makeWindowActive();

    void mouseMoveEvent(QMouseEvent *event);
    void keyPressEvent(QKeyEvent *event);

    void mouseReleaseEvent(QMouseEvent *event);
    void mousePressEvent(QMouseEvent *event);
    void wheelEvent(QWheelEvent *event);

    void setLastRenderChild( X3DChildNode *n )
    {
      last_render_child = n;
    }

    void SetWidth(int width);
    void SetHeight(int height);

  public slots:
     inline void updateScene()
     {
    	 for( set< Scene * >::iterator i = Scene::scenes.begin(); i != Scene::scenes.end(); i++ )
    		 if( (*i)->isActive() )
    			 (*i)->idle();

     }

  protected:
    void initializeGL();
    void paintGL();
    void resizeGL(int width, int height);

    /// The H3DNodeDatabase for this node.
    static H3DNodeDatabase database;


  };
}

#endif // GL_WINDOW_NODE_H
