/**
 * PartitionModelTest.h
 *
 *  Created on: Feb 24, 2009
 *      Author: barrygr
 */

#ifndef PARTITIONMODELTEST_H_
#define PARTITIONMODELTEST_H_

#include <cppunit/TestFixture.h>
#include <cppunit/extensions/HelperMacros.h>
#include "../src/PartitionModel.h"
#include <H3D/X3D.h>

using namespace FS;
using namespace H3D;
using namespace X3D;

class PartitionModelTest : public CppUnit::TestFixture
{
	CPPUNIT_TEST_SUITE( PartitionModelTest );
	CPPUNIT_TEST( test );
	CPPUNIT_TEST_SUITE_END();
private:

public:
  void setUp()
  {
  }

  void tearDown()
  {
  }

  void test()
  {
	  string xmlString;

	  // test that exception is thrown if segmentIdField is missing
	  xmlString = "<PartitionModel DEF='PM'/>";
	  CPPUNIT_ASSERT_THROW(H3D::X3D::createX3DNodeFromString(xmlString), PartitionModel::PartitionModelException);

	  // test that exception is thrown if UNDEFINED_PARTITION_ID is used for another partition
	  stringstream xmlStream1;
	  xmlStream1 << "<PartitionModel DEF='PM'" <<
		  " partitionIdField = '" << (int)UNDEFINED_PARTITION_ID << "'" <<
		  "></PartitionModel>";
	  CPPUNIT_ASSERT_THROW(H3D::X3D::createX3DNodeFromStream(xmlStream1), PartitionModel::PartitionModelException);

	  // test that exception is thrown if partitionNameField length is not the same as partitionIdField
	  xmlString = "<PartitionModel DEF='PM'"
		  " partitionIdField = '1'"
		  "></PartitionModel>";
	  CPPUNIT_ASSERT_THROW(H3D::X3D::createX3DNodeFromString(xmlString), PartitionModel::PartitionModelException);

	  // test that exception is thrown if UNDEFINED_PARTITION_NAME is used for another partition
	  stringstream xmlStream2;
	  xmlStream2 << "<PartitionModel DEF='PM'" <<
	  	  " partitionIdField = '1'" <<
		  " partitionNameField = '" << UNDEFINED_PARTITION_NAME << "'" <<
		  "></PartitionModel>";
	  CPPUNIT_ASSERT_THROW(H3D::X3D::createX3DNodeFromStream(xmlStream2), PartitionModel::PartitionModelException);

	  // test that no exception is thrown for valid data
	  xmlString = "<PartitionModel DEF='PM'"
		  " partitionIdField = '1'"
		  " partitionNameField = 'Tooth Root'"
		  "></PartitionModel>";
	  CPPUNIT_ASSERT_NO_THROW(H3D::X3D::createX3DNodeFromString(xmlString));

	  // test that we get the name of undefined partition
	  xmlString = xmlString = "<PartitionModel DEF='PM'"
		  " partitionIdField = '3 4'"
		  " partitionNameField ='\"Tooth Root\" \"Tooth Crown\"'"
		  "></PartitionModel>";

	  AutoRef< Node > node = H3D::X3D::createX3DNodeFromString(xmlString);
	  PartitionModel* partitionModel = dynamic_cast<PartitionModel*>(node.get());
	  CPPUNIT_ASSERT(UNDEFINED_PARTITION_NAME == partitionModel->getName(UNDEFINED_PARTITION_ID));

	  // test that we get the name of partition with id 3
	  CPPUNIT_ASSERT("Tooth Root" == partitionModel->getName(3));

	  // test that we get the name of partition with id 4
	  CPPUNIT_ASSERT("Tooth Crown" == partitionModel->getName(4));

	  // test that we get an expection when we have an id that does not exist
	  CPPUNIT_ASSERT_THROW(partitionModel->getName(5), PartitionModel::PartitionModelException);


	  // test that we get the name of the undefined partition
	  CPPUNIT_ASSERT(UNDEFINED_PARTITION_ID == partitionModel->getId(UNDEFINED_PARTITION_NAME));

	  // test that we get the id of partition with Tooth Root
	  CPPUNIT_ASSERT(3 == partitionModel->getId("Tooth Root"));

	  // test that we get the name of partition with id Tooth Crown
	  CPPUNIT_ASSERT(4 == partitionModel->getId("Tooth Crown"));

	  // test that we get an expection when we have an name that does not exist
	  CPPUNIT_ASSERT_THROW(partitionModel->getId("Tooth Slime"), PartitionModel::PartitionModelException);


	  // insure that we get a valid list of segment ids
	  const std::vector<unsigned char> partitionIds = partitionModel->getPartitionIds();
	  CPPUNIT_ASSERT(partitionIds.size() == 3);
	  CPPUNIT_ASSERT(partitionIds[0] == UNDEFINED_PARTITION_ID);
	  CPPUNIT_ASSERT(partitionIds[1] == 3);
	  CPPUNIT_ASSERT(partitionIds[2] == 4);
  }

};

#endif /* PARTITIONMODELTEST_H_ */
