from H3DInterface import *

t, = references.getValue()

class CallbackEventField( TypedField( AutoUpdate( SFBool ), Field ) ):
  def __init__( self, callback_function, args ):
    AutoUpdate( SFBool ).__init__( self )
    self.cb = (callback_function, args )

  def update( self, event ):
    apply( self.cb[0], self.cb[1] )
    return 1

class MouseManager:
  def __init__( self, transform ):
    self.transform = transform
    self.enabled = 0
    self.mouse_sensor = createX3DNodeFromString( "<MouseSensor />" )[0]
    self.motion_cb = CallbackEventField( self.onMotionEvent, () )
    self.mouse_sensor.motion.route( self.motion_cb )
    self.scrollup_cb = CallbackEventField( self.onScrollUpEvent, () )
    self.mouse_sensor.scrollUp.route( self.scrollup_cb )
    self.scrolldown_cb = CallbackEventField( self.onScrollDownEvent, () )
    self.mouse_sensor.scrollDown.route( self.scrolldown_cb )

    self.rotation_sensitivity = 0.001
    self.translation_sensitivity = 0.001
    self.scale_sensitivity = 0.001
    self.rotation_offset = Rotation( 1, 0, 0, 0 )

  def setEnabled( self, enabled ):
    self.enabled = enabled

  def setRotationOffset( self, offset ):
    self.rotation_offset = offset

  def onMotionEvent( self ):
    if( self.enabled ):
      motion = self.mouse_sensor.motion.getValue()    
      if( self.mouse_sensor.leftButton.getValue() ):
        epsilon = 1e-8
        if( motion * motion > epsilon ):
          perp = Vec3f( motion.y, motion.x, 0 )
          perp = self.rotation_offset * perp
          perp.normalize();
          r = self.transform.rotation.getValue()
          self.transform.rotation.setValue(  Rotation( perp, 
                                                       motion.length() * self.rotation_sensitivity ) * r )

      if( self.mouse_sensor.rightButton.getValue() ):
        t = self.transform.translation.getValue()
        self.transform.translation.setValue( t + Vec3f( motion.x, -motion.y, 0 ) * self.translation_sensitivity )
        

  def onScrollUpEvent( self ):
    if( self.enabled ):
      s = self.transform.scale.getValue()
      self.transform.scale.setValue( s * 0.99 )
      print self.transform.scale.getValue()

  def onScrollDownEvent( self ):
    if( self.enabled ):
      s = self.transform.scale.getValue()
      self.transform.scale.setValue( s * 1.01 )
      print self.transform.scale.getValue()

mouse_manager = MouseManager( t )
mouse_manager.setEnabled( 1 )
