//////////////////////////////////////////////////////////////////////////////
//    Copyright 2008 Forsslund Systems AB
//
//    ADrillableNode is free software; you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation; either version 2 of the License, or
//    (at your option) any later version.
//
//    ADrillableNode is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with ADrillableNode; if not, write to the Free Software
//    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
//    If you wish to use the code in a proprietary project, please contact
//    jonas@forsslundsystems.se for more information.
//
//////////////////////////////////////////////////////////////////////////////
#include "ComposableShadedVolumeStyle.h"
#include <H3D/MedX3D/VolumeGradient.h>

#include <H3D/ShaderPart.h>
#include <H3D/PixelTexture.h>
#include <H3D/Pixel3DTexture.h>
#include <H3D/TextureProperties.h>


using namespace FS;

  H3D::H3DNodeDatabase ComposableShadedVolumeStyle::database( "ComposableShadedVolumeStyle",
                               &(newInstance<ComposableShadedVolumeStyle>),
                               typeid( ComposableShadedVolumeStyle ),
                               &X3DComposableVolumeRenderStyleNode::database );

  namespace ComposableShadedVolumeStyleInternals {
  H3D::FIELDDB_ELEMENT( ComposableShadedVolumeStyle, material, INPUT_OUTPUT );
  H3D::FIELDDB_ELEMENT( ComposableShadedVolumeStyle, phaseFunction, INPUT_OUTPUT );
  H3D::FIELDDB_ELEMENT( ComposableShadedVolumeStyle, shadows, INPUT_OUTPUT );
  H3D::FIELDDB_ELEMENT( ComposableShadedVolumeStyle, lighting, INPUT_OUTPUT );
  H3D::FIELDDB_ELEMENT( ComposableShadedVolumeStyle, surfaceNormals, INPUT_OUTPUT );
  }

  ComposableShadedVolumeStyle::
  ComposableShadedVolumeStyle( H3D::Inst< DisplayList >_displayList,
		  H3D::Inst< H3D::SFBool > _enabled,
		  H3D::Inst< H3D::SFNode > _metadata,
		  H3D::Inst< SFMaterialNode > _material,
		  H3D::Inst< H3D::SFString > _phaseFunction,
		  H3D::Inst< H3D::SFBool > _shadows,
		  H3D::Inst< H3D::SFBool > _lighting,
		  H3D::Inst< SFTexture3DNode > _surfaceNormals):
			  H3D::X3DComposableVolumeRenderStyleNode( _displayList, _enabled, _metadata ),
    material( _material ),
    phaseFunction( _phaseFunction ),
    shadows( _shadows ),
    lighting( _lighting ),
    surfaceNormals( _surfaceNormals ),
    had_normals( false )




    {
    type_name = "ComposableShadedVolumeStyle";
    database.initFields( this );

    // defaults
    lighting->setValue( false );
    phaseFunction->setValue( "Henyey-Greenstein" );
    shadows->setValue( false );

    // routings
    material->route( displayList );
    phaseFunction->route( displayList );
    shadows->route( displayList );
    surfaceNormals->route( displayList );
  }

  string ComposableShadedVolumeStyle::addUniforms( H3D::ComposedShader *s ) {

	  return

    // add the fields to the shader
    addUniformToFragmentShader( s,
                                UNIFORM_ID(enabled),
                                "bool",
                                H3D::Field::INPUT_OUTPUT,
                                copyAndRouteField(enabled) ) +
    addUniformToFragmentShader( s,
                                UNIFORM_ID(lighting),
                                "bool",
                                H3D::Field::INPUT_OUTPUT,
                                copyAndRouteField(lighting) ) +
    /*addUniformToFragmentShader( s,
                                UNIFORM_ID(shadows),
                                "bool",
                                H3D::Field::INPUT_OUTPUT,
                                copyAndRouteField(shadows) ) + */
    (surfaceNormals->getValue() ?
      addUniformToFragmentShader( s,
                                  UNIFORM_ID(surfaceNormals),
                                  "sampler3D",
                                  H3D::Field::INPUT_OUTPUT,
                                  copyAndRouteField( surfaceNormals) ) : "")
      ;
  }

  void ComposableShadedVolumeStyle::updateUniformFields( H3D::X3DVolumeNode *vd ) {


	 //cout << "updateUniform..." << endl;


	// rebuild shader if normals have been added or removed since
    // we need to add/remove the uniform surfaceNormals then

    /* HACK. DON'T KNOW IF NEEDED.
    // bool have_normals = surfaceNormals->getValue() != NULL;
    //if( have_normals != had_normals ) rebuildShader->touch();
    had_normals = true;//have_normals;
    */

    bool have_normals = surfaceNormals->getValue() != NULL;
    if( have_normals != had_normals )
    	rebuildShader->touch();
    had_normals = have_normals;


    // render material to make it available in shader
    H3D::X3DMaterialNode *m = material->getValue();
    if ( m ) m->displayList->callList();

    H3D::X3DComposableVolumeRenderStyleNode::updateUniformFields( vd );
  }

  string ComposableShadedVolumeStyle::getShaderCode( ) {
    return
      string( "ComposableShadedVolumeStyle(" ) +
      string( "sample_color, " ) +
      string( "r0, " ) +
      string( "viewdir_tex, " ) +
      string( "view_to_tex, " ) +
      UNIFORM_ID_(enabled) +
      string( "true, " ) +
	  //UNIFORM_ID_(shadows) +
      UNIFORM_ID_(lighting) +
      (surfaceNormals->getValue() ?
       "normalizedNormalFromTexture( " + UNIFORM_ID(surfaceNormals) + ", r0 )" :
       "sample_default_normal" ) +
      ");";
  }
