/*
 * DefaultValues.h
 *
 *  Created on: Feb 18, 2009
 *      Author: jocke
 */

#ifndef DEFAULTVALUES_H_
#define DEFAULTVALUES_H_

#include <H3D/X3DChildNode.h>
#include <H3D/SFBool.h>
#include <H3D/SFFloat.h>
#include <H3D/SFInt32.h>
#include <H3D/MFFloat.h>
#include <H3D/MFString.h>
#include <H3D/SFVec3f.h>
#include "SFUnsignedInt8.h"
#include "MFUnsignedInt8.h"
#include <stdexcept>

namespace FS
{

/**
 * \brief Stores all the defaultvalues set in x3d file.
 */
class DefaultValues: public H3D::X3DChildNode
{

public:

	/**
	 * Constructor
	 * \param partitionIdField A list of Partition Ids
	 * \param partitionNameField A list of Partition Names

	 */
	DefaultValues(
			H3D::Inst< H3D::SFFloat> scaleFactorField = 0,
			H3D::Inst< H3D::SFFloat> translationZoomFactorField = 0,
			H3D::Inst< H3D::SFFloat> rotationZoomFactorField = 0,
			H3D::Inst< H3D::SFInt32> renderModeField = 0,
			H3D::Inst< H3D::SFFloat> drillSpeedMaxField = 0,
			H3D::Inst< H3D::SFFloat> cutRadiusField = 0,
			H3D::Inst< H3D::SFInt32> objectToMoveField = 0
	);

	virtual ~DefaultValues()
	{
		cout << "DefaultValues Model Destroyed" << endl;
	}

	/**
	 * Initialize the DefaultValues Model Node
	 */
	void initialize();

	/**
	 * Traverse the Scene Graph for DefaultValues Model Node
	 */
	void traverseSG(H3D::TraverseInfo &ti);

	/**
	 * The H3DNodeDatabase for this node.
	 */
	static H3D::H3DNodeDatabase database;

	auto_ptr<H3D::SFFloat> scaleFactorField;
	auto_ptr<H3D::SFFloat> translationZoomFactorField;
	auto_ptr<H3D::SFFloat> rotationZoomFactorField;
	auto_ptr<H3D::SFInt32> renderModeField;
	auto_ptr<H3D::SFFloat> drillSpeedMaxField;
	auto_ptr<H3D::SFFloat> cutRadiusField;
	auto_ptr<H3D::SFInt32> objectToMoveField;

private:

	/**
	 * \brief Exception class for Partition Model
	 */
	class DefaultValuesException: public runtime_error
	{
	public:

		/**
		 * Constructor
		 */
		DefaultValuesException(const string& what) :
			runtime_error("Exception within DefaultValues: " + what)
		{
		}
	};
};

}

#endif /* DEFAULTVALUES_H_ */
