//////////////////////////////////////////////////////////////////////////////
//    Copyright 2008 Forsslund Systems AB
//
//    ADrillableNode is free software; you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation; either version 2 of the License, or
//    (at your option) any later version.
//
//    ADrillableNode is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with ADrillableNode; if not, write to the Free Software
//    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
//    If you wish to use the code in a proprietary project, please contact
//    jonas@forsslundsystems.se for more information.
//
//////////////////////////////////////////////////////////////////////////////

#ifndef _HIGHLIGHTNODE_H
#define	_HIGHLIGHTNODE_H
#include <H3D/H3DForceEffect.h>
#include <H3D/SFVec3f.h>
#include <H3D/MFFloat.h>
#include "AHapticDrillForce.h"
#include <stdexcept>
#include "VolumeModel.h"
#include <H3DUtil/Image.h>
#include <H3DUtil/TimeStamp.h>
#include <teem/nrrd.h>
#include "DrillingStepsNode.h"

namespace FS
{
    /**
     * Used to Highlight certain areas of the volume. For example, we may wish to highlight where
     * the expert has bored for a particular drilling step
     */
	class HighlightNode: public H3D::X3DChildNode
    {
    public:

        /**
         * Constructor
         * \param A trigger to update the highlighting image.
         * \param The volume model node
         * \param The drilling steps node
         */
        HighlightNode(H3D::Inst< H3D::SFNode  > volumeModelField = 0,
        		H3D::Inst< H3D::SFNode  > drillingStepsField = 0);

        virtual ~HighlightNode()
        {
        	cout<<"Highlight Node Destroyed"<<endl;
        }

        virtual void initialize();
        virtual void traverseSG( H3D::TraverseInfo &ti );

        /*
         *
         * The volume model node
         */
        auto_ptr< H3D::SFNode > volumeModelField;

        /*
         * The drilling steps node
         */
        auto_ptr< H3D::SFNode > drillingStepsField;


        /// The H3DNodeDatabase for this node.
        static H3D::H3DNodeDatabase database;

        /*
         * Highlight regions
         */
        void highlightUser(bool enable);
        void highlightExpert(bool enable);
        void highlightForbidden(bool enable);


    	/**
    	 * Exception class for HighlightNode
    	 */
    	class HighlightNodeException: public runtime_error
    	{
    	public:

    		/**
    		 * Constructor
    		 */
    		HighlightNodeException(const string& what) :
    			runtime_error("Exception within HighlightNode: " + what)
    			{
    			}
    	};

    private:

    	VolumeModel *volumeModelNode;
    	DrillingStepsNode* drillingStepsNode;
    	unsigned char currentDrillingStep;
    	bool highlightExpertOn;

    	enum highlightType_e { NORMAL, USER, EXPERT, FORBIDDEN, USER_VS_EXPERT_VS_FORBIDDEN };

    	highlightType_e highlightType;	// what we are currently highlighting
    };
}

#endif	/* _HIGHLIGHTNODE_H */

