#include "MaterialSegmentationModel.h"

using namespace FS;

/// Add this node to the H3DNodeDatabase system.
H3D::H3DNodeDatabase MaterialSegmentationModel::database("MaterialSegmentationModel",
		&(newInstance<MaterialSegmentationModel>),
		typeid( MaterialSegmentationModel ),
		&X3DChildNode::database );

namespace MaterialSegmentationModelInternals
{
H3D::FIELDDB_ELEMENT( MaterialSegmentationModel, segmentNameField, 				INPUT_OUTPUT );
H3D::FIELDDB_ELEMENT( MaterialSegmentationModel, segmentHardnessField, 		INPUT_OUTPUT );
H3D::FIELDDB_ELEMENT(MaterialSegmentationModel, noOfVoxelsBoredByUser, OUTPUT_ONLY);
}

MaterialSegmentationModel::MaterialSegmentationModel(
		H3D::Inst< H3D::MFString  > segmentNameField,
		H3D::Inst< H3D::MFFloat  > segmentHardnessField,
		H3D::Inst< H3D::MFInt32 > 				noOfVoxelsBoredByUser)
:segmentNameField(segmentNameField),
segmentHardnessField(segmentHardnessField),
noOfVoxelsBoredByUser(noOfVoxelsBoredByUser)
{
			cout << "Material Segmentation Model Constructing"<<endl;
			type_name = "MaterialSegmentationModel";
			database.initFields( this );
			cout << "Material Segmentation Model Constructed"<<endl;
}

		MaterialSegmentationModel::~MaterialSegmentationModel()
		{
			cout<<"Material Segmentation Model Destroyed"<<endl;
		}

		void MaterialSegmentationModel::initialize()
		{
			cout<<"Material Segmentation Model Initializing"<<endl;

			// check if the number of segments defined within the segmentIdField is greater than zero
			if(!(segmentNameField->getValue()).size())
				throw MaterialSegmentationModelException("segmentNameField should contain one or more values");

			if(segmentHardnessField->getSize() != segmentNameField->getSize())
				throw MaterialSegmentationModelException("segmentNameField should contain the same number of values as segmentHardnessField");


			// initialize the noOfVoxelsBoredByUser list with zeros
			unsigned int size = (segmentNameField->getValue()).size();
			H3DInt32* data = new H3DInt32[size];
			for(unsigned int i = 0; i< size;i++)
				data[i] = 0;
			noOfVoxelsBoredByUser->setValueFromVoidPtr(data,size,size*sizeof(H3DInt32),this->id);
			delete[] data;

			X3DChildNode::initialize();


			cout<<"Material Segmentation Model Initialized"<<endl;
		}




		void MaterialSegmentationModel::incrementNoOfVoxelsBoredByUser(const unsigned char& segmentId)
		{
			if(getNoOfSegments()>=segmentId)
			{
				unsigned int currentValue = noOfVoxelsBoredByUser->getValue(this->id)[segmentId];
				noOfVoxelsBoredByUser->setValue(segmentId, ++currentValue,this->id);
			}
			else
				{
					throw MaterialSegmentationModelException("MaterialSegmentationModel::incrementNoOfVoxelseBoredByUser  segmentId out of bounds");
				}

			}



		void MaterialSegmentationModel::setHardness(const unsigned int segmentId, const float& hardness)
		{
			if(getNoOfSegments()>=segmentId)
			{
				segmentHardnessField->setValue(segmentId, hardness);
				cout<<"Material hardness for segment with Id: " << (int)segmentId <<" set to "<< hardness<<endl;
			}
			else
			{
				throw MaterialSegmentationModelException("getHardness segmentId out of bounds");
			}
		}

		const float MaterialSegmentationModel::getHardness(const unsigned int segmentId)
		{
			if(getNoOfSegments()>=segmentId)
			{
				return segmentHardnessField->getValue()[segmentId];
			}
			else
			{
				throw MaterialSegmentationModelException("getHardness segmentId out of bounds");
			}
		}

		const string MaterialSegmentationModel::getName(const unsigned int segmentId)
		{
			if(getNoOfSegments()>=segmentId)
			{
				return segmentNameField->getValue()[segmentId];
			}
			else
			{
				throw MaterialSegmentationModelException("getHardness segmentId out of bounds");
			}
		}



		const std::vector<unsigned char> MaterialSegmentationModel::getSegmentIds()
		{
			vector<unsigned char> segmentIds;

			for(int i=0;i < getNoOfSegments();i++)
				segmentIds.push_back(i);
			return segmentIds;
		}

		const unsigned int MaterialSegmentationModel::getNoOfSegments()
		{
			return segmentNameField->getSize();	// add one for air
		}
