/**
 * MouseManager.cpp
 *
 *  Created on: Mar 31, 2009
 *      Author: barrygr
 */

#include "MouseManager.h"

using namespace FS;

/// Add this node to the H3DNodeDatabase system.
H3D::H3DNodeDatabase MouseManager::database( "MouseManager",
		&(newInstance<MouseManager>),
		typeid( MouseManager ),
		&X3DChildNode::database );

namespace MouseManagerInternals
{
H3D::FIELDDB_ELEMENT( MouseManager, transformField, INPUT_ONLY );
}

MouseManager::MouseManager(H3D::Inst< H3D::SFNode > transformField=0):
transformField( transformField ),
rotationSensitivity(0.001),
translationSensitivity(0.001),
scaleSensitivity(0.001)
{
	type_name = "MouseManager";
	database.initFields( this );
}

MouseManager::~MouseManager()
{
}

void MouseManager::initialize()
{
    rotation = Rotation( 1, 0, 0, 0 );

    X3DChildNode::initialize();
}

void MouseManager::mouseMotionAction (int x, int y)
{
	// execute base functionality first
	MouseSensor::mouseMotionAction(x,y);

	if( enabled->getValue(this->id) )
	{
		Vec2f motion_ = motion->getValue(this->id);

		// moving mouse while holding down left mouse button
		if( leftButton->getValue(this->id) )
		{
			float epsilon = 1e-8;
			if( motion_ * motion_ > epsilon )
			{
				Vec3f perp = Vec3f( motion_.y, motion_.x, 0 );
				perp = rotation * perp;
				perp.normalize();

				Transform* transform = static_cast< Transform* >(transformField->getValue(this->id));
				Rotation r = transform->rotation->getValue(this->id);
				transform->rotation->setValue(Rotation( perp, motion_.length() * rotationSensitivity ) * r ,this->id);
			}
		}

		// moving mouse while holding down right mouse button
		if( rightButton->getValue(this->id) )
		{
			Transform* transform = static_cast< Transform* >(transformField->getValue(this->id));
			Vec3f t = transform->translation->getValue(this->id);
			transform->translation->setValue( t + Vec3f( motion_.x, -motion_.y, 0 ) * translationSensitivity );
		}
	}
}

void MouseManager::mouseWheelAction(int direction)
{
	// execute base functionality first
	MouseSensor::mouseWheelAction(direction);

	if( enabled->getValue(this->id) )
	{
		// wheel towards
		if(direction == TOWARDS)
		{
			Transform* transform = static_cast< Transform* >(transformField->getValue(this->id));
			Vec3f s = transform->scale->getValue(this->id);
			transform->scale->setValue( s * 1.01 );
		}
		// wheel away
		else if(direction == FROM)
		{
			Transform* transform = static_cast< Transform* >(transformField->getValue(this->id));
			Vec3f s = transform->scale->getValue(this->id);
			transform->scale->setValue( s * 0.99 );
		}
	}
}
