/**
 * MouseManager.h
 *
 *  Created on: Mar 31, 2009
 *      Author: barrygr
 */

#ifndef MOUSEMANAGER_H_
#define MOUSEMANAGER_H_

#include <H3D/MouseSensor.h>
#include <H3D/SFNode.h>
#include <H3DUtil/Vec3f.h>
#include <H3DUtil/Vec2f.h>
#include <H3DUtil/Rotationd.h>
#include <H3D/Transform.h>
#include <H3DUtil/H3DBasicTypes.h>

namespace FS
{

using namespace std;
using namespace H3D;

/**
 * Manages the rotation, scaling, and translation of the scene via the Mouse using the transform
 * containing the main scene. This code is simply that of ../python/mouse_test.py. The problem with this script
 * was that it never gave up its reference to the scene transform, resulting in destructors not being called at a
 * later stage.
 */
class MouseManager : public H3D::MouseSensor
{
public:
	MouseManager(H3D::Inst< H3D::SFNode > transformField);
	virtual ~MouseManager();
	virtual void initialize();

	/**
	 * The H3DNodeDatabase for this node.
	 */
	static H3D::H3DNodeDatabase database;

	/*
	 * Reference to the scene transform
	 */
	auto_ptr< H3D::SFNode > transformField;

	/**
	 * When the mouse is moved over the scene
	 */
	void mouseMotionAction (int x, int y);

	/**
	 * When the mouse wheel is adjusted
	 */
	void mouseWheelAction(int direction);

private:
	const H3DFloat rotationSensitivity;
	const H3DFloat translationSensitivity;
	const H3DFloat scaleSensitivity;
	Rotation rotation;
};
}

#endif /* MOUSEMANAGER_H_ */
