//////////////////////////////////////////////////////////////////////////////
//    Copyright 2008 Forsslund Systems AB
//
//    SaveVolume is free software; you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation; either version 2 of the License, or
//    (at your option) any later version.
//
//    SaveVolume is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SaveVolume; if not, write to the Free Software
//    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
//    If you wish to use the code in a proprietary project, please contact
//    jonas@forsslundsystems.se for more information.
//
//////////////////////////////////////////////////////////////////////////////
#include "SaveVolume.h"

using namespace FS;


H3D::H3DNodeDatabase SaveVolume::database( "SaveVolume",
		&(newInstance<SaveVolume>),
		typeid( SaveVolume ),
		&H3D::H3DForceEffect::database );

namespace SaveVolumeInternals
{
H3D::FIELDDB_ELEMENT( SaveVolume, volumeModel, INITIALIZE_ONLY );
H3D::FIELDDB_ELEMENT( SaveVolume, isActive,  INPUT_OUTPUT  );
H3D::FIELDDB_ELEMENT( SaveVolume, fileName,  INPUT_OUTPUT  );
}


SaveVolume::SaveVolume( H3D::Inst< H3D::SFNode  > _volumeModel,
		H3D::Inst < H3D::SFBool > _isActive,
		H3D::Inst < H3D::SFString > _fileName):
			volumeModel( _volumeModel ),
			isActive( _isActive),
			fileName( _fileName )  {

	type_name = "SaveVolume";
	database.initFields( this );

	/* Default values */
	isActive->setValue(false);
	fileName->setValue("defaultTimestampFile");
}

void SaveVolume::initialize()
{
	cout<<"Initializing Save Volume Node"<<endl;

	volume_model = static_cast< VolumeModel * >( volumeModel->getValue() );
	if(!volume_model)
	{
		//TODO:Replace by throw
		cout << "Null pointer exception in volume model!\n";
		return;
	}

	X3DChildNode::initialize();
}


void SaveVolume::traverseSG( H3D::TraverseInfo &ti )
{
	if(isActive->getValue())
	{
		volume_model->saveTimeStamps();
		isActive->setValue(false);
	}
}
