//////////////////////////////////////////////////////////////////////////////
//    Copyright 2008 Forsslund Systems AB
//
//    SaveVolume is free software; you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation; either version 2 of the License, or
//    (at your option) any later version.
//
//    SaveVolume is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SaveVolume; if not, write to the Free Software
//    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
//    If you wish to use the code in a proprietary project, please contact
//    jonas@forsslundsystems.se for more information.
//
//////////////////////////////////////////////////////////////////////////////

#ifndef _SaveVolume_H
#define	_SaveVolume_H
#include <H3D/H3DForceEffect.h>
#include <H3D/SFVec3f.h>
#include "AHapticDrillForce.h"
#include "VolumeModel.h"


namespace FS
{
/** \brief Saves the timestamps of current drill scenario to a nrrd-file.
 *
 * When isActive is set (by user interaction) a volume of timestamps is saved to a nrrd file.
 * Filename can be set by 'fileName' variable.
 */
class SaveVolume: public H3D::X3DChildNode
{
public:

	/// The SFTextureNode field is dependent on the displayList field
	/// of the containing X3DTextureNode node.
	typedef H3D::DependentSFNode< H3D::X3DTexture3DNode,
	H3D::FieldRef< H3D::H3DDisplayListObject,
	H3D::H3DDisplayListObject::DisplayList,
	&H3D::H3DDisplayListObject::displayList >,
	true >// VolumeData::SFTexture3DNode
	SFTexture3DNode;

	/**
	 * Constructor
	 * \param volumeModel A pointer to the volume within which we bore plus its associated properties. These properties
     * include for example the segment to which each voxel belongs.
     * \param isActive When isActive is set (by user interaction) a volume of timestamps is saved to a nrrd file.
     * \param fileName Filename of timestampfile to be saved.
	 */
	SaveVolume( H3D::Inst< H3D::SFNode  > _voluemModel = 0,
			H3D::Inst< H3D::SFBool > _isActive = 0,
			H3D::Inst< H3D::SFString > _fileName = 0);

	/**
	 * SaveVolume Node initialization
	 */
	void initialize();

	/**
	 * Traverse Scene Graph
	 *
	 * Saves the timestamps to a nrrd file if isActive is True.
	 */
	virtual void traverseSG( H3D::TraverseInfo &ti );

	auto_ptr< H3D::SFNode > volumeModel;
	auto_ptr< H3D::SFBool > isActive;
	auto_ptr< H3D::SFString > fileName;

	VolumeModel *volume_model;

	/// The H3DNodeDatabase for this node.
	static H3D::H3DNodeDatabase database;

	/**
	 * Exception class for SaveVolume
	 */
	class SaveVolumeException: public runtime_error
	{
	public:

		/**
		 * Constructor
		 */
		SaveVolumeException(const string& what) :
			runtime_error("Exception within SaveVolume: " + what)
			{
			}
	};
};
}




#endif	/* _SaveVolume_H */

