/**
 * StateMachine.cpp
 *
 *  Created on: March 25, 2009
 *      Author: parhe
 */

#include "StateMachine.h"

using namespace std;
using namespace FS;

/// Add this node to the H3DNodeDatabase system.
H3D::H3DNodeDatabase StateMachine::database("StateMachine",
		&(newInstance<StateMachine>),
		typeid( StateMachine ),
		&X3DChildNode::database );

namespace StateMachineInternals
{
H3D::FIELDDB_ELEMENT( StateMachine, triggerPointsRequiredField, INPUT_OUTPUT );
H3D::FIELDDB_ELEMENT( StateMachine, distancesRequiredField, INPUT_OUTPUT );
H3D::FIELDDB_ELEMENT( StateMachine, forcesRequiredField, INPUT_OUTPUT );
H3D::FIELDDB_ELEMENT( StateMachine, deviceShapesRequiredField, INPUT_OUTPUT );
H3D::FIELDDB_ELEMENT( StateMachine, fractionOfExpertRequiredField, INPUT_OUTPUT );
H3D::FIELDDB_ELEMENT( StateMachine, rotationAnglesRequiredField, INPUT_OUTPUT );
H3D::FIELDDB_ELEMENT( StateMachine, partitionNamesField, INPUT_OUTPUT );
H3D::FIELDDB_ELEMENT( StateMachine, drillingStepsField, INPUT_OUTPUT );
H3D::FIELDDB_ELEMENT( StateMachine, currentDeviceShapeField, INPUT_OUTPUT );
H3D::FIELDDB_ELEMENT( StateMachine, currentForceField, INPUT_OUTPUT );
H3D::FIELDDB_ELEMENT( StateMachine, drillPosField, INPUT_OUTPUT );
H3D::FIELDDB_ELEMENT( StateMachine, currentRotationAnglesField, INPUT_OUTPUT );
H3D::FIELDDB_ELEMENT( StateMachine, stateField, INPUT_OUTPUT );
H3D::FIELDDB_ELEMENT( StateMachine, volumeModelField,INPUT_OUTPUT);
H3D::FIELDDB_ELEMENT( StateMachine, drillingStepsNodeField, INPUT_OUTPUT );
H3D::FIELDDB_ELEMENT( StateMachine, stateObjectivesField, INPUT_OUTPUT );
}

StateMachine::StateMachine(H3D::Inst< H3D::MFVec3f  > _triggerPointsRequiredField,
		H3D::Inst< H3D::MFFloat> _distancesRequiredField,
		H3D::Inst< H3D::MFFloat> _forcesRequiredField,
		H3D::Inst< H3D::MFInt32> _deviceShapesRequiredField,
		H3D::Inst< H3D::MFFloat> _fractionOfExpertRequiredField,
		H3D::Inst< H3D::MFFloat> _rotationAnglesRequiredField,
		H3D::Inst< H3D::MFString> _partitionNamesField,
		H3D::Inst< H3D::MFInt32> _drillingStepsField,
		H3D::Inst< H3D::SFInt32> _currentDeviceShapeField,
		H3D::Inst< H3D::SFVec3f> _currentForceField,
		H3D::Inst< H3D::SFVec3f> _drillPosField,
		H3D::Inst< H3D::SFVec3f> _currentRotationAnglesField,
		H3D::Inst< H3D::SFInt32> _stateField,
		H3D::Inst< H3D::SFNode > _volumeModelField,
		H3D::Inst< H3D::SFNode > _drillingStepsNodeField,
		H3D::Inst< H3D::MFString > _stateObjectivesField

):
	triggerPointsRequiredField(_triggerPointsRequiredField),
	distancesRequiredField(_distancesRequiredField),
	forcesRequiredField(_forcesRequiredField),
	deviceShapesRequiredField(_deviceShapesRequiredField),
	fractionOfExpertRequiredField(_fractionOfExpertRequiredField),
	rotationAnglesRequiredField(_rotationAnglesRequiredField),
	partitionNamesField(_partitionNamesField),
	drillingStepsField(_drillingStepsField),
	currentDeviceShapeField(_currentDeviceShapeField),
	currentForceField(_currentForceField),
	drillPosField(_drillPosField),
	currentRotationAnglesField(_currentRotationAnglesField),
	stateField(_stateField),
	volumeModelField(_volumeModelField),
	drillingStepsNodeField(_drillingStepsNodeField),
	stateObjectivesField(_stateObjectivesField)
	{
	cout << "StateMachine Model Constructing"<<endl;

	type_name = "StateMachine";
	database.initFields( this );
	cout << "StateMachine Model Constructed"<<endl;



	}



void StateMachine::initialize()
{
	cout<<"StateMachine Model Initializing"<<endl;
	absoluteForce.setValue(0.0);
	distance.setValue(0.0);
	rotationAngle.setValue(0.0);
	initialRotationAngle.setValue(fabs(currentRotationAnglesField->getValue().z));
	currentDrillingStep = 1; //0= all voxels that aren't included in any drillingstep
	volumeModelNode = static_cast< VolumeModel * >( volumeModelField->getValue() );
	drillingStepsNode = static_cast< DrillingStepsNode * >( drillingStepsNodeField->getValue() );

	if(!volumeModelNode)
	{
		cout << "Null pointer exception in volume model!\n";
		return;
	}

	if(!drillingStepsNode)
	{
		cout << "Null pointer exception in drilling steps node!\n";
		return;
	}

	maxNrOfStates = triggerPointsRequiredField->getSize();

	//Check that all arrays have the same length
	if (triggerPointsRequiredField->getSize()!=maxNrOfStates){
		throw StateMachineException("Number of states excised in triggerPointsRequiredField");
	}

	if (distancesRequiredField->getSize()!=maxNrOfStates){
		throw StateMachineException("Number of states excised in distancesRequiredField");
	}

	if (forcesRequiredField->getSize()!=maxNrOfStates){
		throw StateMachineException("Number of states excised in forcesRequiredField");
	}

	if (deviceShapesRequiredField->getSize()!=maxNrOfStates){
		throw StateMachineException("Number of states excised in deviceShapesRequiredField");
	}

	if (fractionOfExpertRequiredField->getSize()!=maxNrOfStates){
		throw StateMachineException("Number of states excised in fractionOfExpertRequiredField");
	}

	if (rotationAnglesRequiredField->getSize()!=maxNrOfStates){
		throw StateMachineException("Number of states excised in rotationAnglesRequiredField");
	}

	if (partitionNamesField->getSize()!=maxNrOfStates){
		throw StateMachineException("Number of states excised in partitionNamesField");
	}

	if (drillingStepsField->getSize()!=maxNrOfStates){
		throw StateMachineException("Number of states excised in drillingStepsField");
	}



	X3DChildNode::initialize();

	cout<<"StateMachine Model Initialized"<<endl;
}

void StateMachine::traverseSG( H3D::TraverseInfo &ti )
{

	unsigned int currentState = stateField->getValue();

	//Check if the required fields has values for the current state and request the value.
	if (currentState == maxNrOfStates){
		return; //Nothing to do already completed the last state
	}
	else{

		if (maxNrOfStates<currentState)
		{
			throw StateMachineException("Number of states excised in traverseSG");
		}

		distance.setValue((drillPosField->getValue()-triggerPointsRequiredField->getValue()[currentState]).length());
		absoluteForce.setValue((currentForceField->getValue()).length());

		//Both rotation and initial angle is zero i.e set initial to current angle
		if(rotationAngle.getValue()==0&&initialRotationAngle.getValue()==0){
			initialRotationAngle.setValue(fabs(currentRotationAnglesField->getValue().z));
		}
		else{

			//Check if the rotation is increasing from the initial angle.
			//If not reset the initial position and set the total rotation to zero
			if (rotationAngle.getValue()<= fabs(fabs(currentRotationAnglesField->getValue().z)-fabs(initialRotationAngle.getValue())))
			{
				//rotationAngle should be positive!
				rotationAngle.setValue(fabs(fabs(currentRotationAnglesField->getValue().z)-fabs(initialRotationAngle.getValue())));
			}
			else
			{
				//Rotation decreases i.e reset start angle and rotation = 0
				initialRotationAngle.setValue(fabs(currentRotationAnglesField->getValue().z));
				rotationAngle.setValue(0);
			}
		}

		//Check if the distance is less than the threshold if not return
		if((distancesRequiredField->getValue()[currentState])<distance.getValue()){
			return;
		}

		//Check if the force is above the threshold if not return
		if((forcesRequiredField->getValue()[currentState])>absoluteForce.getValue()){
			return;
		}

		//Check if the device shape is the correct one, if not return
		if(deviceShapesRequiredField->getValue()[currentState] != currentDeviceShapeField->getValue())
		{
			return;
		}

		//Check if the fraction drilled is above than the threshold if not return
		if(fractionOfExpertRequiredField->getValue()[currentState] >drillingStepsNode->fractionOfVoxelsDrilledByUserToExpertForCurrentDrillingStep())
		{
			return;
		}

		//Now check if the rotation is larger than the required value
		if(rotationAnglesRequiredField->getValue()[currentState]>rotationAngle.getValue())
		{
			return;
		}

		//reset whatever needs to be reseted before entering the next state
		resetState();

		//Passed all checks, increment current state and drilling step
		nextState();
	}
}


//Resets and prepare a state
void StateMachine::resetState(){
	initialRotationAngle.setValue(0);
	rotationAngle.setValue(0);
}


void StateMachine::nextState(){

	cout<<"Entering Next State currentState: "<<stateField->getValue()<<endl;

	unsigned int currentState = stateField->getValue(); //initialized to 1

	if (maxNrOfStates<currentState)
	{
		throw StateMachineException("Number of states excised in nextState");
	}

	if (maxNrOfStates==currentState){
		//Do nothing already completed the last state
	}
	else
	{
		if (partitionNamesField->getValue()[currentState]!=""){
			volumeModelNode->removePartitionByName(partitionNamesField->getValue()[currentState]);
		}


		if (drillingStepsField->getValue()[currentState]>currentDrillingStep){
			currentDrillingStep++;
			drillingStepsNode->incrementCurrentDrillingStep();
		}
		stateField->setValue(currentState+1); //increment the current state
	}

	cout<<"Exiting Next State currentState: "<<stateField->getValue()<<endl;
}

SFFloat StateMachine::getDistance()
{
	return distance;
}

SFFloat StateMachine::getAbsoluteForce()
{
	return absoluteForce;
}

